import typing
from typing import Union

import re
import uuid
from collections import Counter

import numpy as np
import pandas as pd
from scipy.stats import chi2, ks_2samp
from scipy.stats.stats import Ks_2sampResult, wasserstein_distance

from giskard.ml_worker.core.giskard_dataset import GiskardDataset
from giskard.ml_worker.core.model import GiskardModel
from giskard.ml_worker.generated.ml_worker_pb2 import SingleTestResult, TestMessage, TestMessageType
from giskard.ml_worker.testing.abstract_test_collection import AbstractTestCollection


class DriftTests(AbstractTestCollection):
    # Class Variable
    other_modalities_pattern = "^other_modalities_[a-z0-9]{32}$"

    @staticmethod
    def _calculate_psi(category, actual_distribution, expected_distribution):
        # To use log and avoid zero distribution probability,
        # we bound distribution probability by min_distribution_probability
        min_distribution_probability = 0.0001

        expected_distribution_bounded = max(
            expected_distribution[category], min_distribution_probability
        )
        actual_distribution_bounded = max(
            actual_distribution[category], min_distribution_probability
        )
        modality_psi = (expected_distribution_bounded - actual_distribution_bounded) * np.log(
            expected_distribution_bounded / actual_distribution_bounded
        )
        return modality_psi

    @staticmethod
    def _calculate_frequencies(actual_series, reference_series, max_categories=None):
        all_modalities = list(set(reference_series).union(set(actual_series)))
        if max_categories is not None and len(all_modalities) > max_categories:
            var_count_expected = dict(Counter(reference_series).most_common(max_categories))
            other_modalities_key = "other_modalities_" + uuid.uuid1().hex
            var_count_expected[other_modalities_key] = len(reference_series) - sum(
                var_count_expected.values()
            )
            categories_list = list(var_count_expected.keys())

            var_count_actual = Counter(actual_series)
            # For test data, we take the same category names as expected_data
            var_count_actual = {i: var_count_actual[i] for i in categories_list}
            var_count_actual[other_modalities_key] = len(actual_series) - sum(
                var_count_actual.values()
            )

            all_modalities = categories_list
        else:
            var_count_expected = Counter(reference_series)
            var_count_actual = Counter(actual_series)
        expected_frequencies = np.array([var_count_expected[i] for i in all_modalities])
        actual_frequencies = np.array([var_count_actual[i] for i in all_modalities])
        return all_modalities, actual_frequencies, expected_frequencies

    @staticmethod
    def _calculate_drift_psi(actual_series, reference_series, max_categories):
        (
            all_modalities,
            actual_frequencies,
            expected_frequencies,
        ) = DriftTests._calculate_frequencies(actual_series, reference_series, max_categories)
        expected_distribution = expected_frequencies / len(reference_series)
        actual_distribution = actual_frequencies / len(actual_series)
        total_psi = 0
        output_data = pd.DataFrame(
            columns=["Modality", "Reference_distribution", "Actual_distribution", "Psi"]
        )
        for category in range(len(all_modalities)):
            modality_psi = DriftTests._calculate_psi(
                category, actual_distribution, expected_distribution
            )

            total_psi += modality_psi
            row = {
                "Modality": all_modalities[category],
                "Reference_distribution": expected_distribution[category],
                "Actual_distribution": expected_distribution[category],
                "Psi": modality_psi,
            }

            output_data = output_data.append(pd.Series(row), ignore_index=True)
        return total_psi, output_data

    @staticmethod
    def _calculate_ks(actual_series, reference_series) -> Ks_2sampResult:
        return ks_2samp(reference_series, actual_series)

    @staticmethod
    def _calculate_earth_movers_distance(actual_series, reference_series):
        unique_reference = np.unique(reference_series)
        unique_actual = np.unique(actual_series)
        sample_space = list(set(unique_reference).union(set(unique_actual)))
        val_max = max(sample_space)
        val_min = min(sample_space)
        if val_max == val_min:
            metric = 0
        else:
            # Normalizing reference_series and actual_series for comparison purposes
            reference_series = (reference_series - val_min) / (val_max - val_min)
            actual_series = (actual_series - val_min) / (val_max - val_min)

            metric = wasserstein_distance(reference_series, actual_series)
        return metric

    @staticmethod
    def _calculate_chi_square(actual_series, reference_series, max_categories):
        (
            all_modalities,
            actual_frequencies,
            expected_frequencies,
        ) = DriftTests._calculate_frequencies(actual_series, reference_series, max_categories)
        chi_square = 0
        # it's necessary for comparison purposes to normalize expected_frequencies
        # so that reference and actual has the same size
        # See https://github.com/scipy/scipy/blob/v1.8.0/scipy/stats/_stats_py.py#L6787
        k_norm = actual_series.shape[0] / reference_series.shape[0]
        output_data = pd.DataFrame(
            columns=["Modality", "Reference_frequencies", "Actual_frequencies", "Chi_square"]
        )
        for i in range(len(all_modalities)):
            chi_square_value = (actual_frequencies[i] - expected_frequencies[i] * k_norm) ** 2 / (
                expected_frequencies[i] * k_norm
            )
            chi_square += chi_square_value

            row = {
                "Modality": all_modalities[i],
                "Reference_frequencies": expected_frequencies[i],
                "Actual_frequencies": actual_frequencies[i],
                "Chi_square": chi_square_value,
            }

            output_data = output_data.append(pd.Series(row), ignore_index=True)
        # if reference_series and actual_series has only one modality it turns nan (len(all_modalities)=1)
        if len(all_modalities) > 1:
            chi_cdf = chi2.cdf(chi_square, len(all_modalities) - 1)
            p_value = 1 - chi_cdf if chi_cdf != 0 else 0
        else:
            p_value = 0
        return chi_square, p_value, output_data

    @staticmethod
    def _validate_column_type(gsk_dataset, column_name, column_type):
        assert gsk_dataset.feature_types[column_name] == column_type, (
            f'Column "{column_name}" is not of type "{column_type}"'
        )

    @staticmethod
    def _validate_column_name(actual_ds, reference_ds, column_name):
        assert (
            column_name in actual_ds.columns
        ), f'"{column_name}" is not a column of Actual Dataset Columns: {", ".join(actual_ds.columns)}'
        assert (
            column_name in reference_ds.columns
        ), f'"{column_name}" is not a column of Reference Dataset Columns: {", ".join(reference_ds.columns)}'

    @staticmethod
    def _validate_series_notempty(actual_series, reference_series):
        if actual_series.empty:
            raise ValueError("Actual Series computed from the column is empty")
        if reference_series.empty:
            raise ValueError("Reference Series computed from the column is empty")

    def _extract_series(self, actual_ds, reference_ds, column_name, column_type):
        actual_ds.df.reset_index(drop=True, inplace=True)
        reference_ds.df.reset_index(drop=True, inplace=True)
        self._validate_column_name(actual_ds, reference_ds, column_name)
        self._validate_column_type(actual_ds, column_name, column_type)
        self._validate_column_type(reference_ds, column_name, column_type)
        actual_series = actual_ds.df[column_name]
        reference_series = reference_ds.df[column_name]
        self._validate_series_notempty(actual_series, reference_series)
        return actual_series, reference_series

    def test_drift_psi(
        self,
        reference_ds: GiskardDataset,
        actual_ds: GiskardDataset,
        column_name: str,
        threshold=0.2,
        max_categories: int = 20,
        psi_contribution_percent: float = 0.2,
    ) -> SingleTestResult:
        """
        Test if the PSI score between the actual and expected datasets is below the threshold for
        a given categorical feature

        Example : The test is passed when the  PSI score of gender between reference and actual sets is below 0.2

        Args:
            actual_ds(GiskardDataset):
                Actual dataset to compute the test
            reference_ds(GiskardDataset):
                Reference dataset to compute the test
            column_name(str):
                Name of column with categorical feature
            threshold(float:
                Threshold value for PSI
            max_categories:
                the maximum categories to compute the PSI score
            psi_contribution_percent:
                the ratio between the PSI score of a given category over the total PSI score
                of the categorical variable. If there is a drift, the test provides all the
                categories that have a PSI contribution over than this ratio.

        Returns:
            actual_slices_size:
                Length of rows with given categorical feature in actual slice
            reference_slices_size:
                Length of rows with given categorical feature in reference slice
            metric:
                The total psi score between the actual and expected datasets
            passed:
                TRUE if total_psi <= threshold
        """
        actual_series, reference_series = self._extract_series(
            actual_ds, reference_ds, column_name, "category"
        )

        messages, passed, total_psi = self._test_series_drift_psi(
            actual_series,
            reference_series,
            "data",
            max_categories,
            psi_contribution_percent,
            threshold,
        )

        return self.save_results(
            SingleTestResult(
                actual_slices_size=[len(actual_series)],
                reference_slices_size=[len(reference_series)],
                passed=passed,
                metric=total_psi,
                messages=messages,
            )
        )

    def test_drift_chi_square(
        self,
        reference_ds: GiskardDataset,
        actual_ds: GiskardDataset,
        column_name: str,
        threshold=0.05,
        max_categories: int = 20,
        chi_square_contribution_percent: float = 0.2,
    ) -> SingleTestResult:
        """
        Test if the p-value of the chi square test between the actual and expected datasets is
        above the threshold for a given categorical feature

        Example : The test is passed when the pvalue of the chi square test of the categorical variable between
         reference and actual sets is higher than 0.05. It means that chi square test cannot be rejected at 5% level
         and that we cannot assume drift for this variable.

        Args:
            actual_ds(GiskardDataset):
                Actual dataset to compute the test
            reference_ds(GiskardDataset):
                Reference dataset to compute the test
            column_name(str):
                Name of column with categorical feature
            threshold(float):
                Threshold for p-value of chi-square
            max_categories:
                the maximum categories to compute the chi square
            chi_square_contribution_percent:
                the ratio between the Chi-Square value of a given category over the total Chi-Square
                value of the categorical variable. If there is a drift, the test provides all the
                categories that have a PSI contribution over than this ratio.

        Returns:
            actual_slices_size:
                Length of rows with given categorical feature in actual slice
            reference_slices_size:
                Length of rows with given categorical feature in reference slice
            metric:
                The pvalue of chi square test
            passed:
                TRUE if metric > threshold
        """
        actual_series, reference_series = self._extract_series(
            actual_ds, reference_ds, column_name, "category"
        )

        messages, p_value, passed = self._test_series_drift_chi(
            actual_series,
            reference_series,
            "data",
            chi_square_contribution_percent,
            max_categories,
            threshold,
        )

        return self.save_results(
            SingleTestResult(
                actual_slices_size=[len(actual_series)],
                reference_slices_size=[len(reference_series)],
                passed=passed,
                metric=p_value,
                messages=messages,
            )
        )

    def test_drift_ks(
        self,
        reference_ds: GiskardDataset,
        actual_ds: GiskardDataset,
        column_name: str,
        threshold=0.05,
    ) -> SingleTestResult:
        """
        Test if the pvalue of the KS test between the actual and expected datasets is above
        the threshold for a given numerical feature

        Example : The test is passed when the pvalue of the KS test of the numerical variable
        between the actual and expected datasets is higher than 0.05. It means that the KS test
        cannot be rejected at 5% level and that we cannot assume drift for this variable.

        Args:
            actual_ds(GiskardDataset):
               Actual dataset to compute the test
            reference_ds(GiskardDataset):
                Reference dataset to compute the test
            column_name(str):
                Name of column with numerical feature
            threshold:
                Threshold for p-value of KS test

        Returns:
            actual_slices_size:
                Length of rows with given numerical feature in actual slice
            reference_slices_size:
                Length of rows with given numerical feature in reference slice
            metric:
                The pvalue of KS test
            passed:
                TRUE if metric >= threshold
        """
        actual_series, reference_series = self._extract_series(
            actual_ds, reference_ds, column_name, "numeric"
        )

        result = self._calculate_ks(actual_series, reference_series)

        passed = result.pvalue >= threshold

        messages = self._generate_message_ks(passed, result, threshold, "data")

        return self.save_results(
            SingleTestResult(
                actual_slices_size=[len(actual_series)],
                reference_slices_size=[len(reference_series)],
                passed=passed,
                metric=result.pvalue,
                messages=messages,
            )
        )

    def test_drift_earth_movers_distance(
        self,
        reference_ds: GiskardDataset,
        actual_ds: GiskardDataset,
        column_name: str,
        threshold: float = 0.2,
    ) -> SingleTestResult:
        """
        Test if the earth movers distance between the actual and expected datasets is
        below the threshold for a given numerical feature

        Example : The test is passed when the earth movers distance of the numerical
         variable between the actual and expected datasets is lower than 0.1.
         It means that we cannot assume drift for this variable.

        Args:
            actual_ds(GiskardDataset):
                Actual dataset to compute the test
            reference_ds(GiskardDataset):
                Reference dataset to compute the test
            column_name(str):
                Name of column with numerical feature
            threshold:
                Threshold for earth movers distance

        Returns:
            actual_slices_size:
                Length of rows with given numerical feature in actual slice
            reference_slices_size:
                Length of rows with given numerical feature in reference slice
            metric:
                The earth movers distance
            passed:
                TRUE if metric <= threshold
        """
        actual_series, reference_series = self._extract_series(
            actual_ds, reference_ds, column_name, "numeric"
        )

        metric = self._calculate_earth_movers_distance(actual_series, reference_series)

        passed = metric <= threshold

        messages: Union[typing.List[TestMessage], None] = None

        if not passed:
            messages = [
                TestMessage(
                    type=TestMessageType.ERROR,
                    text=f"The data is drifting (metric is equal to {np.round(metric, 9)} and is below the test risk level {threshold}) ",
                )
            ]
        return self.save_results(
            SingleTestResult(
                actual_slices_size=[len(actual_series)],
                reference_slices_size=[len(reference_series)],
                passed=True if threshold is None else passed,
                metric=metric,
                messages=messages,
            )
        )

    def test_drift_prediction_psi(
        self,
        reference_slice: GiskardDataset,
        actual_slice: GiskardDataset,
        model: GiskardModel,
        max_categories: int = 10,
        threshold: float = 0.2,
        psi_contribution_percent: float = 0.2,
    ):
        """
        Test if the PSI score between the reference and actual datasets is below the threshold
        for the classification labels predictions

        Example : The test is passed when the  PSI score of classification labels prediction
        for females between reference and actual sets is below 0.2

        Args:
            actual_slice(GiskardDataset):
                Slice of the actual dataset
            reference_slice(GiskardDataset):
                Slice of the reference dataset
            model(GiskardModel):
                Model used to compute the test
            threshold(float):
                Threshold value for PSI
            max_categories:
                The maximum categories to compute the PSI score
            psi_contribution_percent:
                The ratio between the PSI score of a given category over the total PSI score
                of the categorical variable. If there is a drift, the test provides all the
                categories that have a PSI contribution over than this ratio.

        Returns:
            actual_slices_size:
                Length of actual slice tested
            reference_slices_size:
                Length of reference slice tested
            passed:
                TRUE if metric <= threshold
            metric:
                Total PSI value
            messages:
                Psi result message
        """
        actual_slice.df.reset_index(drop=True, inplace=True)
        reference_slice.df.reset_index(drop=True, inplace=True)
        prediction_reference = pd.Series(model.run_predict(reference_slice).prediction)
        prediction_actual = pd.Series(model.run_predict(actual_slice).prediction)
        messages, passed, total_psi = self._test_series_drift_psi(
            prediction_actual,
            prediction_reference,
            "prediction",
            max_categories,
            psi_contribution_percent,
            threshold,
        )

        return self.save_results(
            SingleTestResult(
                actual_slices_size=[len(actual_slice)],
                reference_slices_size=[len(reference_slice)],
                passed=passed,
                metric=total_psi,
                messages=messages,
            )
        )

    def _test_series_drift_psi(
        self,
        actual_series,
        reference_series,
        test_data,
        max_categories,
        psi_contribution_percent,
        threshold,
    ):
        total_psi, output_data = self._calculate_drift_psi(
            actual_series, reference_series, max_categories
        )
        passed = True if threshold is None else total_psi <= threshold
        main_drifting_modalities_bool = output_data["Psi"] > psi_contribution_percent * total_psi
        messages = self._generate_message_modalities(
            main_drifting_modalities_bool, output_data, test_data
        )
        return messages, passed, total_psi

    @staticmethod
    def _generate_message_modalities(main_drifting_modalities_bool, output_data, test_data):
        modalities_list = output_data[main_drifting_modalities_bool]["Modality"].tolist()
        filtered_modalities = [
            w for w in modalities_list if not re.match(DriftTests.other_modalities_pattern, w)
        ]
        messages: Union[typing.List[TestMessage], None] = None
        if filtered_modalities:
            messages = [
                TestMessage(
                    type=TestMessageType.ERROR,
                    text=f"The {test_data} is drifting for the following modalities: {','.join(filtered_modalities)}",
                )
            ]
        return messages

    def test_drift_prediction_chi_square(
        self,
        reference_slice: GiskardDataset,
        actual_slice: GiskardDataset,
        model: GiskardModel,
        max_categories: int = 10,
        threshold: float = 0.05,
        chi_square_contribution_percent: float = 0.2,
    ):
        """
        Test if the Chi Square value between the reference and actual datasets is below the threshold
        for the classification labels predictions for a given slice

        Example : The test is passed when the  Chi Square value of classification labels prediction
        for females between reference and actual sets is below 0.2

        Args:
            actual_slice(GiskardDataset):
                Slice of the actual dataset
            reference_slice(GiskardDataset):
                Slice of the reference dataset
            model(GiskardModel):
                Model used to compute the test
            threshold(float):
                Threshold value of p-value of Chi-Square
            max_categories:
                the maximum categories to compute the PSI score
            chi_square_contribution_percent:
                the ratio between the Chi-Square value of a given category over the total Chi-Square
                value of the categorical variable. If there is a drift, the test provides all the
                categories that have a PSI contribution over than this ratio.

        Returns:
            actual_slices_size:
                Length of actual slice tested
            reference_slices_size:
                Length of reference slice tested
            passed:
                TRUE if metric > threshold
            metric:
                Calculated p-value of Chi_square
            messages:
                Message describing if prediction is drifting or not
        """
        actual_slice.df.reset_index(drop=True, inplace=True)
        reference_slice.df.reset_index(drop=True, inplace=True)
        prediction_reference = pd.Series(model.run_predict(reference_slice).prediction)
        prediction_actual = pd.Series(model.run_predict(actual_slice).prediction)

        messages, p_value, passed = self._test_series_drift_chi(
            prediction_actual,
            prediction_reference,
            "prediction",
            chi_square_contribution_percent,
            max_categories,
            threshold,
        )

        return self.save_results(
            SingleTestResult(
                actual_slices_size=[len(actual_slice)],
                reference_slices_size=[len(reference_slice)],
                passed=passed,
                metric=p_value,
                messages=messages,
            )
        )

    def _test_series_drift_chi(
        self,
        actual_series,
        reference_series,
        test_data,
        chi_square_contribution_percent,
        max_categories,
        threshold,
    ):
        chi_square, p_value, output_data = self._calculate_chi_square(
            actual_series, reference_series, max_categories
        )
        passed = p_value > threshold
        main_drifting_modalities_bool = (
            output_data["Chi_square"] > chi_square_contribution_percent * chi_square
        )
        messages = self._generate_message_modalities(
            main_drifting_modalities_bool, output_data, test_data
        )
        return messages, p_value, passed

    def test_drift_prediction_ks(
        self,
        reference_slice: GiskardDataset,
        actual_slice: GiskardDataset,
        model: GiskardModel,
        classification_label=None,
        threshold=None,
    ) -> SingleTestResult:
        """
        Test if the pvalue of the KS test for prediction between the reference and actual datasets for
         a given subpopulation is above the threshold

        Example : The test is passed when the pvalue of the KS test for the prediction for females
         between reference and actual dataset is higher than 0.05. It means that the KS test cannot be
         rejected at 5% level and that we cannot assume drift for this variable.

        Args:
            actual_slice(GiskardDataset):
                Slice of the actual dataset
            reference_slice(GiskardDataset):
                Slice of the reference dataset
            model(GiskardModel):
                Model used to compute the test
            threshold(float):
                Threshold for p-value of Kolmogorov-Smirnov test
            classification_label(str):
                One specific label value from the target column for classification model

        Returns:
            actual_slices_size:
                Length of actual slice tested
            reference_slices_size:
                Length of reference slice tested
            passed:
                TRUE if metric >= threshold
            metric:
                The calculated p-value Kolmogorov-Smirnov test
            messages:
                Kolmogorov-Smirnov result message
        """
        actual_slice.df.reset_index(drop=True, inplace=True)
        reference_slice.df.reset_index(drop=True, inplace=True)

        assert (
            model.model_type != "classification"
            or classification_label in model.classification_labels
        ), f'"{classification_label}" is not part of model labels: {",".join(model.classification_labels)}'

        prediction_reference = (
            pd.Series(
                model.run_predict(reference_slice).all_predictions[classification_label].values
            )
            if model.model_type == "classification"
            else pd.Series(model.run_predict(reference_slice).prediction)
        )
        prediction_actual = (
            pd.Series(model.run_predict(actual_slice).all_predictions[classification_label].values)
            if model.model_type == "classification"
            else pd.Series(model.run_predict(actual_slice).prediction)
        )

        result: Ks_2sampResult = self._calculate_ks(prediction_reference, prediction_actual)

        passed = True if threshold is None else result.pvalue >= threshold

        messages = self._generate_message_ks(passed, result, threshold, "prediction")

        return self.save_results(
            SingleTestResult(
                actual_slices_size=[len(actual_slice)],
                reference_slices_size=[len(reference_slice)],
                passed=passed,
                metric=result.pvalue,
                messages=messages,
            )
        )

    @staticmethod
    def _generate_message_ks(passed, result, threshold, data_type):
        messages: Union[typing.List[TestMessage], None] = None
        if not passed:
            messages = [
                TestMessage(
                    type=TestMessageType.ERROR,
                    text=f"The {data_type} is drifting (p-value is equal to {np.round(result.pvalue, 9)} "
                    f"and is below the test risk level {threshold}) ",
                )
            ]
        return messages

    def test_drift_prediction_earth_movers_distance(
        self,
        reference_slice: GiskardDataset,
        actual_slice: GiskardDataset,
        model: GiskardModel,
        classification_label=None,
        threshold=0.2,
    ) -> SingleTestResult:
        """
        Test if the Earth Mover’s Distance value between the reference and actual datasets is
        below the threshold for the classification labels predictions for classification
        model and prediction for regression models

        Example :
        Classification : The test is passed when the  Earth Mover’s Distance value of classification
        labels probabilities for females between reference and actual sets is below 0.2

        Regression : The test is passed when the  Earth Mover’s Distance value of prediction
        for females between reference and actual sets is below 0.2

        Args:
            reference_slice(GiskardDataset):
                slice of the reference dataset
            actual_slice(GiskardDataset):
                slice of the actual dataset
            model(GiskardModel):
                uploaded model
            classification_label:
                one specific label value from the target column for classification model
            threshold:
                threshold for earth mover's distance

        Returns:
            passed:
                TRUE if metric <= threshold
            metric:
                Earth Mover's Distance value

        """
        actual_slice.df.reset_index(drop=True, inplace=True)
        reference_slice.df.reset_index(drop=True, inplace=True)

        prediction_reference = (
            model.run_predict(reference_slice).all_predictions[classification_label].values
            if model.model_type == "classification"
            else model.run_predict(reference_slice).prediction
        )
        prediction_actual = (
            model.run_predict(actual_slice).all_predictions[classification_label].values
            if model.model_type == "classification"
            else model.run_predict(actual_slice).prediction
        )

        metric = self._calculate_earth_movers_distance(prediction_reference, prediction_actual)

        passed = True if threshold is None else metric <= threshold
        messages: Union[typing.List[TestMessage], None] = None

        if not passed:
            messages = [
                TestMessage(
                    type=TestMessageType.ERROR,
                    text=f"The prediction is drifting (metric is equal to {np.round(metric, 9)} "
                    f"and is above the test risk level {threshold}) ",
                )
            ]
        return self.save_results(
            SingleTestResult(
                actual_slices_size=[len(actual_slice)],
                reference_slices_size=[len(reference_slice)],
                passed=True if threshold is None else metric <= threshold,
                metric=metric,
                messages=messages,
            )
        )



from typing import Optional

import os
from pathlib import Path

from pydantic import BaseSettings


class Settings(BaseSettings):
    home: str = "~/giskard-home"
    port: int = 50051
    host: str = "localhost"
    max_workers: int = 10
    max_send_message_length_mb: int = 1024
    max_receive_message_length_mb: int = 1024
    loglevel = "INFO"

    class Config:
        env_prefix = "GSK_"


def expand_env_var(env_var: Optional[str]) -> Optional[str]:
    if not env_var:
        return env_var
    while True:
        interpolated = os.path.expanduser(os.path.expandvars(str(env_var)))
        if interpolated == env_var:
            return interpolated
        else:
            env_var = interpolated


settings = Settings()
run_dir = Path(expand_env_var(settings.home)) / "run"



import logging
import os
import platform
import re
import sys

import grpc
import numpy as np
import pandas as pd
import pkg_resources
import psutil
import tqdm
from eli5.lime import TextExplainer

from giskard.ml_worker.core.giskard_dataset import GiskardDataset
from giskard.ml_worker.core.model_explanation import (
    explain,
    parse_text_explainer_response,
    text_explanation_prediction_wrapper,
)
from giskard.ml_worker.exceptions.giskard_exception import GiskardException
from giskard.ml_worker.exceptions.IllegalArgumentError import IllegalArgumentError
from giskard.ml_worker.generated.ml_worker_pb2 import (
    DataFrame,
    DataRow,
    EchoMsg,
    ExplainRequest,
    ExplainResponse,
    ExplainTextRequest,
    ExplainTextResponse,
    FileUploadRequest,
    MLWorkerInfo,
    MLWorkerInfoRequest,
    PlatformInfo,
    RunModelForDataFrameRequest,
    RunModelForDataFrameResponse,
    RunModelRequest,
    RunModelResponse,
    RunTestRequest,
    TestResultMessage,
    UploadStatus,
    UploadStatusCode,
)
from giskard.ml_worker.generated.ml_worker_pb2_grpc import MLWorkerServicer
from giskard.ml_worker.utils.grpc_mapper import deserialize_dataset, deserialize_model
from giskard.ml_worker.utils.logging import Timer

logger = logging.getLogger(__name__)

cnt = 1


class MLWorkerServiceImpl(MLWorkerServicer):
    def __init__(self, port=None, remote=None) -> None:
        super().__init__()
        self.port = port
        self.remote = remote

    def echo(self, request, context):
        globals()["cnt"] += 1
        return EchoMsg(msg=f"Response {cnt}: {request.msg}")

    def upload(self, request_iterator, context):
        upload_message: FileUploadRequest
        progress = None
        for upload_message in request_iterator:
            if upload_message.HasField("metadata"):
                print(upload_message.metadata.name)
                progress = tqdm.tqdm(
                    f"Receiving {upload_message.metadata.name}",
                    unit="B",
                    unit_scale=True,
                    unit_divisor=1024,
                )
            elif upload_message.HasField("chunk"):
                progress.update(len(upload_message.chunk.content))
        progress.close()
        return UploadStatus(code=UploadStatusCode.Ok)

    def getInfo(self, request: MLWorkerInfoRequest, context):
        installed_packages = (
            {p.project_name: p.version for p in pkg_resources.working_set}
            if request.list_packages
            else None
        )
        current_process = psutil.Process(os.getpid())
        return MLWorkerInfo(
            platform=PlatformInfo(
                machine=platform.uname().machine,
                node=platform.uname().node,
                processor=platform.uname().processor,
                release=platform.uname().release,
                system=platform.uname().system,
                version=platform.uname().version,
            ),
            pid=os.getpid(),
            process_start_time=int(current_process.create_time()),
            interpreter=sys.executable,
            interpreter_version=platform.python_version(),
            installed_packages=installed_packages,
            internal_grpc_port=self.port,
            is_remote=self.remote,
        )

    def runTest(self, request: RunTestRequest, context: grpc.ServicerContext) -> TestResultMessage:
        from giskard.ml_worker.testing.functions import GiskardTestFunctions

        model = deserialize_model(request.model)

        tests = GiskardTestFunctions()
        _globals = {"model": model, "tests": tests}
        if request.reference_ds.serialized_df:
            _globals["reference_ds"] = deserialize_dataset(request.reference_ds)
        if request.actual_ds.serialized_df:
            _globals["actual_ds"] = deserialize_dataset(request.actual_ds)
        try:
            timer = Timer()
            exec(request.code, _globals)
            timer.stop(f"Test {tests.tests_results[0].name}")
        except NameError as e:
            missing_name = re.findall(r"name '(\w+)' is not defined", str(e))[0]
            if missing_name == "reference_ds":
                raise IllegalArgumentError("Reference Dataset is not specified")
            if missing_name == "actual_ds":
                raise IllegalArgumentError("Actual Dataset is not specified")
            raise e

        return TestResultMessage(results=tests.tests_results)

    def explain(self, request: ExplainRequest, context) -> ExplainResponse:
        model = deserialize_model(request.model)
        dataset = deserialize_dataset(request.dataset)
        explanations = explain(model, dataset, request.columns)

        return ExplainResponse(
            explanations={
                k: ExplainResponse.Explanation(per_feature=v)
                for k, v in explanations["explanations"].items()
            }
        )

    def explainText(self, request: ExplainTextRequest, context) -> ExplainTextResponse:
        n_samples = 500 if request.n_samples <= 0 else request.n_samples
        model = deserialize_model(request.model)
        text_column = request.feature_name

        if request.feature_types[text_column] != "text":
            raise ValueError(f"Column {text_column} is not of type text")
        text_document = request.columns[text_column]
        input_df = pd.DataFrame({k: [v] for k, v in request.columns.items()})
        if model.feature_names:
            input_df = input_df[model.feature_names]
        text_explainer = TextExplainer(random_state=42, n_samples=n_samples)
        prediction_function = text_explanation_prediction_wrapper(
            model.prediction_function, input_df, text_column
        )
        text_explainer.fit(text_document, prediction_function)
        html_response = text_explainer.show_prediction(
            target_names=model.classification_labels
        )._repr_html_()
        return ExplainTextResponse(explanations=parse_text_explainer_response(html_response))

    def runModelForDataFrame(self, request: RunModelForDataFrameRequest, context):
        model = deserialize_model(request.model)
        ds = GiskardDataset(
            pd.DataFrame([r.columns for r in request.dataframe.rows]),
            target=request.target,
            feature_types=request.feature_types,
            column_types=request.column_types,
        )
        predictions = model.run_predict(ds)
        if model.model_type == "classification":
            return RunModelForDataFrameResponse(
                all_predictions=self.pandas_df_to_proto_df(predictions.all_predictions),
                prediction=predictions.prediction.astype(str),
            )
        else:
            return RunModelForDataFrameResponse(
                prediction=predictions.prediction.astype(str), raw_prediction=predictions.prediction
            )

    def runModel(self, request: RunModelRequest, context) -> RunModelResponse:
        try:
            model = deserialize_model(request.model)
            dataset = deserialize_dataset(request.dataset)
        except ValueError as e:
            if "unsupported pickle protocol" in str(e):
                raise ValueError('Unable to unpickle object, '
                                 'Make sure that Python version of client code is the same as the Python version in ML Worker.'
                                 'To change Python version, please refer to https://docs.giskard.ai/start/guides/configuration'
                                 f'\nOriginal Error: {e}') from e
            raise e
        except ModuleNotFoundError as e:
            raise GiskardException(f"Failed to import '{e.name}'. "
                                   f"Make sure it's installed in the ML Worker environment."
                                   "To install it, refer to https://docs.giskard.ai/start/guides/configuration") from e
        prediction_results = model.run_predict(dataset)

        if model.model_type == "classification":
            results = prediction_results.all_predictions
            labels = {k: v for k, v in enumerate(model.classification_labels)}
            label_serie = dataset.df[dataset.target] if dataset.target else None
            if len(model.classification_labels) > 2 or model.classification_threshold is None:
                preds_serie = prediction_results.all_predictions.idxmax(axis="columns")
                sorted_predictions = np.sort(prediction_results.all_predictions.values)
                abs_diff = pd.Series(
                    sorted_predictions[:, -1] - sorted_predictions[:, -2], name="absDiff"
                )
            else:
                diff = (
                        prediction_results.all_predictions.iloc[:, 1] - model.classification_threshold
                )
                preds_serie = (diff >= 0).astype(int).map(labels).rename("predictions")
                abs_diff = pd.Series(diff.abs(), name="absDiff")
            calculated = pd.concat([preds_serie, label_serie, abs_diff], axis=1)
        else:
            results = pd.Series(prediction_results.prediction)
            preds_serie = results
            if dataset.target and dataset.target in dataset.df.columns:
                target_serie = dataset.df[dataset.target]
                diff = preds_serie - target_serie
                diff_percent = pd.Series(diff / target_serie, name="diffPercent")
                abs_diff = pd.Series(diff.abs(), name="absDiff")
                abs_diff_percent = pd.Series(abs_diff / target_serie, name="absDiffPercent")
                calculated = pd.concat(
                    [preds_serie, target_serie, abs_diff, abs_diff_percent, diff_percent], axis=1
                )
            else:
                calculated = pd.concat([preds_serie], axis=1)

        return RunModelResponse(
            results_csv=results.to_csv(index=False), calculated_csv=calculated.to_csv(index=False)
        )

    @staticmethod
    def pandas_df_to_proto_df(df):
        return DataFrame(rows=[DataRow(columns=r.astype(str).to_dict()) for _, r in df.iterrows()])

    @staticmethod
    def pandas_series_to_proto_series(self, series):
        return



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

import giskard.ml_worker.generated.ml_worker_pb2 as ml__worker__pb2


class MLWorkerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.getInfo = channel.unary_unary(
                '/worker.MLWorker/getInfo',
                request_serializer=ml__worker__pb2.MLWorkerInfoRequest.SerializeToString,
                response_deserializer=ml__worker__pb2.MLWorkerInfo.FromString,
                )
        self.runTest = channel.unary_unary(
                '/worker.MLWorker/runTest',
                request_serializer=ml__worker__pb2.RunTestRequest.SerializeToString,
                response_deserializer=ml__worker__pb2.TestResultMessage.FromString,
                )
        self.runModel = channel.unary_unary(
                '/worker.MLWorker/runModel',
                request_serializer=ml__worker__pb2.RunModelRequest.SerializeToString,
                response_deserializer=ml__worker__pb2.RunModelResponse.FromString,
                )
        self.runModelForDataFrame = channel.unary_unary(
                '/worker.MLWorker/runModelForDataFrame',
                request_serializer=ml__worker__pb2.RunModelForDataFrameRequest.SerializeToString,
                response_deserializer=ml__worker__pb2.RunModelForDataFrameResponse.FromString,
                )
        self.explain = channel.unary_unary(
                '/worker.MLWorker/explain',
                request_serializer=ml__worker__pb2.ExplainRequest.SerializeToString,
                response_deserializer=ml__worker__pb2.ExplainResponse.FromString,
                )
        self.explainText = channel.unary_unary(
                '/worker.MLWorker/explainText',
                request_serializer=ml__worker__pb2.ExplainTextRequest.SerializeToString,
                response_deserializer=ml__worker__pb2.ExplainTextResponse.FromString,
                )
        self.echo = channel.unary_unary(
                '/worker.MLWorker/echo',
                request_serializer=ml__worker__pb2.EchoMsg.SerializeToString,
                response_deserializer=ml__worker__pb2.EchoMsg.FromString,
                )
        self.upload = channel.stream_unary(
                '/worker.MLWorker/upload',
                request_serializer=ml__worker__pb2.FileUploadRequest.SerializeToString,
                response_deserializer=ml__worker__pb2.UploadStatus.FromString,
                )


class MLWorkerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def getInfo(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def runTest(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def runModel(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def runModelForDataFrame(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def explain(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def explainText(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def echo(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def upload(self, request_iterator, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_MLWorkerServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'getInfo': grpc.unary_unary_rpc_method_handler(
                    servicer.getInfo,
                    request_deserializer=ml__worker__pb2.MLWorkerInfoRequest.FromString,
                    response_serializer=ml__worker__pb2.MLWorkerInfo.SerializeToString,
            ),
            'runTest': grpc.unary_unary_rpc_method_handler(
                    servicer.runTest,
                    request_deserializer=ml__worker__pb2.RunTestRequest.FromString,
                    response_serializer=ml__worker__pb2.TestResultMessage.SerializeToString,
            ),
            'runModel': grpc.unary_unary_rpc_method_handler(
                    servicer.runModel,
                    request_deserializer=ml__worker__pb2.RunModelRequest.FromString,
                    response_serializer=ml__worker__pb2.RunModelResponse.SerializeToString,
            ),
            'runModelForDataFrame': grpc.unary_unary_rpc_method_handler(
                    servicer.runModelForDataFrame,
                    request_deserializer=ml__worker__pb2.RunModelForDataFrameRequest.FromString,
                    response_serializer=ml__worker__pb2.RunModelForDataFrameResponse.SerializeToString,
            ),
            'explain': grpc.unary_unary_rpc_method_handler(
                    servicer.explain,
                    request_deserializer=ml__worker__pb2.ExplainRequest.FromString,
                    response_serializer=ml__worker__pb2.ExplainResponse.SerializeToString,
            ),
            'explainText': grpc.unary_unary_rpc_method_handler(
                    servicer.explainText,
                    request_deserializer=ml__worker__pb2.ExplainTextRequest.FromString,
                    response_serializer=ml__worker__pb2.ExplainTextResponse.SerializeToString,
            ),
            'echo': grpc.unary_unary_rpc_method_handler(
                    servicer.echo,
                    request_deserializer=ml__worker__pb2.EchoMsg.FromString,
                    response_serializer=ml__worker__pb2.EchoMsg.SerializeToString,
            ),
            'upload': grpc.stream_unary_rpc_method_handler(
                    servicer.upload,
                    request_deserializer=ml__worker__pb2.FileUploadRequest.FromString,
                    response_serializer=ml__worker__pb2.UploadStatus.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'worker.MLWorker', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class MLWorker(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def getInfo(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/worker.MLWorker/getInfo',
            ml__worker__pb2.MLWorkerInfoRequest.SerializeToString,
            ml__worker__pb2.MLWorkerInfo.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def runTest(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/worker.MLWorker/runTest',
            ml__worker__pb2.RunTestRequest.SerializeToString,
            ml__worker__pb2.TestResultMessage.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def runModel(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/worker.MLWorker/runModel',
            ml__worker__pb2.RunModelRequest.SerializeToString,
            ml__worker__pb2.RunModelResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def runModelForDataFrame(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/worker.MLWorker/runModelForDataFrame',
            ml__worker__pb2.RunModelForDataFrameRequest.SerializeToString,
            ml__worker__pb2.RunModelForDataFrameResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def explain(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/worker.MLWorker/explain',
            ml__worker__pb2.ExplainRequest.SerializeToString,
            ml__worker__pb2.ExplainResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def explainText(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/worker.MLWorker/explainText',
            ml__worker__pb2.ExplainTextRequest.SerializeToString,
            ml__worker__pb2.ExplainTextResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def echo(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/worker.MLWorker/echo',
            ml__worker__pb2.EchoMsg.SerializeToString,
            ml__worker__pb2.EchoMsg.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def upload(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_unary(request_iterator, target, '/worker.MLWorker/upload',
            ml__worker__pb2.FileUploadRequest.SerializeToString,
            ml__worker__pb2.UploadStatus.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ml-worker.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0fml-worker.proto\x12\x06worker\x1a\x19google/protobuf/any.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1bgoogle/protobuf/empty.proto\",\n\x13MLWorkerInfoRequest\x12\x15\n\rlist_packages\x18\x01 \x01(\x08\"\xc3\x02\n\x0cMLWorkerInfo\x12&\n\x08platform\x18\x01 \x01(\x0b\x32\x14.worker.PlatformInfo\x12\x13\n\x0binterpreter\x18\x02 \x01(\t\x12\x1b\n\x13interpreter_version\x18\x03 \x01(\t\x12G\n\x12installed_packages\x18\x04 \x03(\x0b\x32+.worker.MLWorkerInfo.InstalledPackagesEntry\x12\x1a\n\x12internal_grpc_port\x18\x05 \x01(\r\x12\x11\n\tis_remote\x18\x06 \x01(\x08\x12\x0b\n\x03pid\x18\x07 \x01(\r\x12\x1a\n\x12process_start_time\x18\x08 \x01(\x04\x1a\x38\n\x16InstalledPackagesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"r\n\x0cPlatformInfo\x12\x0f\n\x07machine\x18\x01 \x01(\t\x12\x0c\n\x04node\x18\x02 \x01(\t\x12\x11\n\tprocessor\x18\x03 \x01(\t\x12\x0f\n\x07release\x18\x04 \x01(\t\x12\x0e\n\x06system\x18\x05 \x01(\t\x12\x0f\n\x07version\x18\x06 \x01(\t\"n\n\x11\x46ileUploadRequest\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1a.worker.FileUploadMetadataH\x00\x12\x1e\n\x05\x63hunk\x18\x02 \x01(\x0b\x32\r.worker.ChunkH\x00\x42\t\n\x07request\"\"\n\x12\x46ileUploadMetadata\x12\x0c\n\x04name\x18\x01 \x01(\t\"\x16\n\x07\x45\x63hoMsg\x12\x0b\n\x03msg\x18\x01 \x01(\t\"\xd8\x01\n\x0e\x45xplainRequest\x12-\n\x05model\x18\x01 \x01(\x0b\x32\x1e.worker.SerializedGiskardModel\x12\x31\n\x07\x64\x61taset\x18\x02 \x01(\x0b\x32 .worker.SerializedGiskardDataset\x12\x34\n\x07\x63olumns\x18\x03 \x03(\x0b\x32#.worker.ExplainRequest.ColumnsEntry\x1a.\n\x0c\x43olumnsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xd0\x02\n\x12\x45xplainTextRequest\x12-\n\x05model\x18\x01 \x01(\x0b\x32\x1e.worker.SerializedGiskardModel\x12\x14\n\x0c\x66\x65\x61ture_name\x18\x02 \x01(\t\x12\x38\n\x07\x63olumns\x18\x03 \x03(\x0b\x32\'.worker.ExplainTextRequest.ColumnsEntry\x12\x43\n\rfeature_types\x18\x04 \x03(\x0b\x32,.worker.ExplainTextRequest.FeatureTypesEntry\x12\x11\n\tn_samples\x18\x05 \x01(\r\x1a.\n\x0c\x43olumnsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x33\n\x11\x46\x65\x61tureTypesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xb9\x02\n\x0f\x45xplainResponse\x12?\n\x0c\x65xplanations\x18\x01 \x03(\x0b\x32).worker.ExplainResponse.ExplanationsEntry\x1a\x8a\x01\n\x0b\x45xplanation\x12H\n\x0bper_feature\x18\x01 \x03(\x0b\x32\x33.worker.ExplainResponse.Explanation.PerFeatureEntry\x1a\x31\n\x0fPerFeatureEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x02:\x02\x38\x01\x1aX\n\x11\x45xplanationsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x32\n\x05value\x18\x02 \x01(\x0b\x32#.worker.ExplainResponse.Explanation:\x02\x38\x01\"\x8f\x01\n\x13\x45xplainTextResponse\x12\x43\n\x0c\x65xplanations\x18\x01 \x03(\x0b\x32-.worker.ExplainTextResponse.ExplanationsEntry\x1a\x33\n\x11\x45xplanationsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x8d\x01\n\x1cRunModelForDataFrameResponse\x12*\n\x0f\x61ll_predictions\x18\x01 \x01(\x0b\x32\x11.worker.DataFrame\x12\x12\n\nprediction\x18\x02 \x03(\t\x12\x15\n\rprobabilities\x18\x03 \x03(\x02\x12\x16\n\x0eraw_prediction\x18\x04 \x03(\x02\"h\n\x07\x44\x61taRow\x12-\n\x07\x63olumns\x18\x01 \x03(\x0b\x32\x1c.worker.DataRow.ColumnsEntry\x1a.\n\x0c\x43olumnsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"*\n\tDataFrame\x12\x1d\n\x04rows\x18\x01 \x03(\x0b\x32\x0f.worker.DataRow\"\x85\x03\n\x1bRunModelForDataFrameRequest\x12-\n\x05model\x18\x01 \x01(\x0b\x32\x1e.worker.SerializedGiskardModel\x12$\n\tdataframe\x18\x02 \x01(\x0b\x32\x11.worker.DataFrame\x12\x0e\n\x06target\x18\x03 \x01(\t\x12L\n\rfeature_types\x18\x04 \x03(\x0b\x32\x35.worker.RunModelForDataFrameRequest.FeatureTypesEntry\x12J\n\x0c\x63olumn_types\x18\x05 \x03(\x0b\x32\x34.worker.RunModelForDataFrameRequest.ColumnTypesEntry\x1a\x33\n\x11\x46\x65\x61tureTypesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x32\n\x10\x43olumnTypesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"s\n\x0fRunModelRequest\x12-\n\x05model\x18\x01 \x01(\x0b\x32\x1e.worker.SerializedGiskardModel\x12\x31\n\x07\x64\x61taset\x18\x02 \x01(\x0b\x32 .worker.SerializedGiskardDataset\"?\n\x10RunModelResponse\x12\x13\n\x0bresults_csv\x18\x01 \x01(\t\x12\x16\n\x0e\x63\x61lculated_csv\x18\x02 \x01(\t\"\xba\x01\n\x0eRunTestRequest\x12\x0c\n\x04\x63ode\x18\x01 \x01(\t\x12-\n\x05model\x18\x02 \x01(\x0b\x32\x1e.worker.SerializedGiskardModel\x12\x33\n\tactual_ds\x18\x03 \x01(\x0b\x32 .worker.SerializedGiskardDataset\x12\x36\n\x0creference_ds\x18\x04 \x01(\x0b\x32 .worker.SerializedGiskardDataset\"\xbe\x02\n\x18SerializedGiskardDataset\x12\x0e\n\x06target\x18\x02 \x01(\t\x12I\n\rfeature_types\x18\x03 \x03(\x0b\x32\x32.worker.SerializedGiskardDataset.FeatureTypesEntry\x12G\n\x0c\x63olumn_types\x18\x05 \x03(\x0b\x32\x31.worker.SerializedGiskardDataset.ColumnTypesEntry\x12\x15\n\rserialized_df\x18\x04 \x01(\x0c\x1a\x33\n\x11\x46\x65\x61tureTypesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x32\n\x10\x43olumnTypesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x1f\n\x0fRunTestResponse\x12\x0c\n\x04name\x18\x01 \x01(\t\"9\n\x19Partial_unexpected_counts\x12\r\n\x05value\x18\x01 \x03(\r\x12\r\n\x05\x63ount\x18\x02 \x01(\r\"O\n\x15NamedSingleTestResult\x12\x0c\n\x04name\x18\x01 \x01(\t\x12(\n\x06result\x18\x02 \x01(\x0b\x32\x18.worker.SingleTestResult\"B\n\x0bTestMessage\x12%\n\x04type\x18\x01 \x01(\x0e\x32\x17.worker.TestMessageType\x12\x0c\n\x04text\x18\x02 \x01(\t\"\x98\x06\n\x10SingleTestResult\x12\x0e\n\x06passed\x18\r \x01(\x08\x12%\n\x08messages\x18\x10 \x03(\x0b\x32\x13.worker.TestMessage\x12\x32\n\x05props\x18\x0e \x03(\x0b\x32#.worker.SingleTestResult.PropsEntry\x12\x0e\n\x06metric\x18\x0f \x01(\x02\x12\x32\n\rmissing_count\x18\x02 \x01(\x0b\x32\x1b.google.protobuf.Int32Value\x12\x35\n\x0fmissing_percent\x18\x03 \x01(\x0b\x32\x1c.google.protobuf.DoubleValue\x12\x35\n\x10unexpected_count\x18\x04 \x01(\x0b\x32\x1b.google.protobuf.Int32Value\x12\x38\n\x12unexpected_percent\x18\x05 \x01(\x0b\x32\x1c.google.protobuf.DoubleValue\x12>\n\x18unexpected_percent_total\x18\x06 \x01(\x0b\x32\x1c.google.protobuf.DoubleValue\x12\x43\n\x1dunexpected_percent_nonmissing\x18\x07 \x01(\x0b\x32\x1c.google.protobuf.DoubleValue\x12%\n\x1dpartial_unexpected_index_list\x18\t \x03(\r\x12\x44\n\x19partial_unexpected_counts\x18\n \x03(\x0b\x32!.worker.Partial_unexpected_counts\x12\x1d\n\x15unexpected_index_list\x18\x0c \x03(\r\x12\x11\n\toutput_df\x18\x12 \x01(\x0c\x12 \n\x18number_of_perturbed_rows\x18\x14 \x01(\r\x12\x1a\n\x12\x61\x63tual_slices_size\x18\x15 \x03(\r\x12\x1d\n\x15reference_slices_size\x18\x16 \x03(\r\x1a,\n\nPropsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"C\n\x11TestResultMessage\x12.\n\x07results\x18\x01 \x03(\x0b\x32\x1d.worker.NamedSingleTestResult\"\xbb\x01\n\x16SerializedGiskardModel\x12&\n\x1eserialized_prediction_function\x18\x01 \x01(\x0c\x12\x12\n\nmodel_type\x18\x02 \x01(\t\x12/\n\tthreshold\x18\x03 \x01(\x0b\x32\x1c.google.protobuf.DoubleValue\x12\x15\n\rfeature_names\x18\x04 \x03(\t\x12\x1d\n\x15\x63lassification_labels\x18\x05 \x03(\t\"\x18\n\x05\x43hunk\x12\x0f\n\x07\x63ontent\x18\x01 \x01(\x0c\"6\n\x0cUploadStatus\x12&\n\x04\x63ode\x18\x01 \x01(\x0e\x32\x18.worker.UploadStatusCode*&\n\x0fTestMessageType\x12\t\n\x05\x45RROR\x10\x00\x12\x08\n\x04INFO\x10\x01*3\n\x10UploadStatusCode\x12\x0b\n\x07Unknown\x10\x00\x12\x06\n\x02Ok\x10\x01\x12\n\n\x06\x46\x61iled\x10\x02\x32\xa3\x04\n\x08MLWorker\x12>\n\x07getInfo\x12\x1b.worker.MLWorkerInfoRequest\x1a\x14.worker.MLWorkerInfo\"\x00\x12>\n\x07runTest\x12\x16.worker.RunTestRequest\x1a\x19.worker.TestResultMessage\"\x00\x12?\n\x08runModel\x12\x17.worker.RunModelRequest\x1a\x18.worker.RunModelResponse\"\x00\x12\x63\n\x14runModelForDataFrame\x12#.worker.RunModelForDataFrameRequest\x1a$.worker.RunModelForDataFrameResponse\"\x00\x12<\n\x07\x65xplain\x12\x16.worker.ExplainRequest\x1a\x17.worker.ExplainResponse\"\x00\x12H\n\x0b\x65xplainText\x12\x1a.worker.ExplainTextRequest\x1a\x1b.worker.ExplainTextResponse\"\x00\x12*\n\x04\x65\x63ho\x12\x0f.worker.EchoMsg\x1a\x0f.worker.EchoMsg\"\x00\x12=\n\x06upload\x12\x19.worker.FileUploadRequest\x1a\x14.worker.UploadStatus\"\x00(\x01\x42(\n\x11\x61i.giskard.workerB\x0bWorkerProtoP\x01\xa2\x02\x03WRKb\x06proto3')

_TESTMESSAGETYPE = DESCRIPTOR.enum_types_by_name['TestMessageType']
TestMessageType = enum_type_wrapper.EnumTypeWrapper(_TESTMESSAGETYPE)
_UPLOADSTATUSCODE = DESCRIPTOR.enum_types_by_name['UploadStatusCode']
UploadStatusCode = enum_type_wrapper.EnumTypeWrapper(_UPLOADSTATUSCODE)
ERROR = 0
INFO = 1
Unknown = 0
Ok = 1
Failed = 2


_MLWORKERINFOREQUEST = DESCRIPTOR.message_types_by_name['MLWorkerInfoRequest']
_MLWORKERINFO = DESCRIPTOR.message_types_by_name['MLWorkerInfo']
_MLWORKERINFO_INSTALLEDPACKAGESENTRY = _MLWORKERINFO.nested_types_by_name['InstalledPackagesEntry']
_PLATFORMINFO = DESCRIPTOR.message_types_by_name['PlatformInfo']
_FILEUPLOADREQUEST = DESCRIPTOR.message_types_by_name['FileUploadRequest']
_FILEUPLOADMETADATA = DESCRIPTOR.message_types_by_name['FileUploadMetadata']
_ECHOMSG = DESCRIPTOR.message_types_by_name['EchoMsg']
_EXPLAINREQUEST = DESCRIPTOR.message_types_by_name['ExplainRequest']
_EXPLAINREQUEST_COLUMNSENTRY = _EXPLAINREQUEST.nested_types_by_name['ColumnsEntry']
_EXPLAINTEXTREQUEST = DESCRIPTOR.message_types_by_name['ExplainTextRequest']
_EXPLAINTEXTREQUEST_COLUMNSENTRY = _EXPLAINTEXTREQUEST.nested_types_by_name['ColumnsEntry']
_EXPLAINTEXTREQUEST_FEATURETYPESENTRY = _EXPLAINTEXTREQUEST.nested_types_by_name['FeatureTypesEntry']
_EXPLAINRESPONSE = DESCRIPTOR.message_types_by_name['ExplainResponse']
_EXPLAINRESPONSE_EXPLANATION = _EXPLAINRESPONSE.nested_types_by_name['Explanation']
_EXPLAINRESPONSE_EXPLANATION_PERFEATUREENTRY = _EXPLAINRESPONSE_EXPLANATION.nested_types_by_name['PerFeatureEntry']
_EXPLAINRESPONSE_EXPLANATIONSENTRY = _EXPLAINRESPONSE.nested_types_by_name['ExplanationsEntry']
_EXPLAINTEXTRESPONSE = DESCRIPTOR.message_types_by_name['ExplainTextResponse']
_EXPLAINTEXTRESPONSE_EXPLANATIONSENTRY = _EXPLAINTEXTRESPONSE.nested_types_by_name['ExplanationsEntry']
_RUNMODELFORDATAFRAMERESPONSE = DESCRIPTOR.message_types_by_name['RunModelForDataFrameResponse']
_DATAROW = DESCRIPTOR.message_types_by_name['DataRow']
_DATAROW_COLUMNSENTRY = _DATAROW.nested_types_by_name['ColumnsEntry']
_DATAFRAME = DESCRIPTOR.message_types_by_name['DataFrame']
_RUNMODELFORDATAFRAMEREQUEST = DESCRIPTOR.message_types_by_name['RunModelForDataFrameRequest']
_RUNMODELFORDATAFRAMEREQUEST_FEATURETYPESENTRY = _RUNMODELFORDATAFRAMEREQUEST.nested_types_by_name['FeatureTypesEntry']
_RUNMODELFORDATAFRAMEREQUEST_COLUMNTYPESENTRY = _RUNMODELFORDATAFRAMEREQUEST.nested_types_by_name['ColumnTypesEntry']
_RUNMODELREQUEST = DESCRIPTOR.message_types_by_name['RunModelRequest']
_RUNMODELRESPONSE = DESCRIPTOR.message_types_by_name['RunModelResponse']
_RUNTESTREQUEST = DESCRIPTOR.message_types_by_name['RunTestRequest']
_SERIALIZEDGISKARDDATASET = DESCRIPTOR.message_types_by_name['SerializedGiskardDataset']
_SERIALIZEDGISKARDDATASET_FEATURETYPESENTRY = _SERIALIZEDGISKARDDATASET.nested_types_by_name['FeatureTypesEntry']
_SERIALIZEDGISKARDDATASET_COLUMNTYPESENTRY = _SERIALIZEDGISKARDDATASET.nested_types_by_name['ColumnTypesEntry']
_RUNTESTRESPONSE = DESCRIPTOR.message_types_by_name['RunTestResponse']
_PARTIAL_UNEXPECTED_COUNTS = DESCRIPTOR.message_types_by_name['Partial_unexpected_counts']
_NAMEDSINGLETESTRESULT = DESCRIPTOR.message_types_by_name['NamedSingleTestResult']
_TESTMESSAGE = DESCRIPTOR.message_types_by_name['TestMessage']
_SINGLETESTRESULT = DESCRIPTOR.message_types_by_name['SingleTestResult']
_SINGLETESTRESULT_PROPSENTRY = _SINGLETESTRESULT.nested_types_by_name['PropsEntry']
_TESTRESULTMESSAGE = DESCRIPTOR.message_types_by_name['TestResultMessage']
_SERIALIZEDGISKARDMODEL = DESCRIPTOR.message_types_by_name['SerializedGiskardModel']
_CHUNK = DESCRIPTOR.message_types_by_name['Chunk']
_UPLOADSTATUS = DESCRIPTOR.message_types_by_name['UploadStatus']
MLWorkerInfoRequest = _reflection.GeneratedProtocolMessageType('MLWorkerInfoRequest', (_message.Message,), {
  'DESCRIPTOR' : _MLWORKERINFOREQUEST,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.MLWorkerInfoRequest)
  })
_sym_db.RegisterMessage(MLWorkerInfoRequest)

MLWorkerInfo = _reflection.GeneratedProtocolMessageType('MLWorkerInfo', (_message.Message,), {

  'InstalledPackagesEntry' : _reflection.GeneratedProtocolMessageType('InstalledPackagesEntry', (_message.Message,), {
    'DESCRIPTOR' : _MLWORKERINFO_INSTALLEDPACKAGESENTRY,
    '__module__' : 'ml_worker_pb2'
    # @@protoc_insertion_point(class_scope:worker.MLWorkerInfo.InstalledPackagesEntry)
    })
  ,
  'DESCRIPTOR' : _MLWORKERINFO,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.MLWorkerInfo)
  })
_sym_db.RegisterMessage(MLWorkerInfo)
_sym_db.RegisterMessage(MLWorkerInfo.InstalledPackagesEntry)

PlatformInfo = _reflection.GeneratedProtocolMessageType('PlatformInfo', (_message.Message,), {
  'DESCRIPTOR' : _PLATFORMINFO,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.PlatformInfo)
  })
_sym_db.RegisterMessage(PlatformInfo)

FileUploadRequest = _reflection.GeneratedProtocolMessageType('FileUploadRequest', (_message.Message,), {
  'DESCRIPTOR' : _FILEUPLOADREQUEST,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.FileUploadRequest)
  })
_sym_db.RegisterMessage(FileUploadRequest)

FileUploadMetadata = _reflection.GeneratedProtocolMessageType('FileUploadMetadata', (_message.Message,), {
  'DESCRIPTOR' : _FILEUPLOADMETADATA,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.FileUploadMetadata)
  })
_sym_db.RegisterMessage(FileUploadMetadata)

EchoMsg = _reflection.GeneratedProtocolMessageType('EchoMsg', (_message.Message,), {
  'DESCRIPTOR' : _ECHOMSG,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.EchoMsg)
  })
_sym_db.RegisterMessage(EchoMsg)

ExplainRequest = _reflection.GeneratedProtocolMessageType('ExplainRequest', (_message.Message,), {

  'ColumnsEntry' : _reflection.GeneratedProtocolMessageType('ColumnsEntry', (_message.Message,), {
    'DESCRIPTOR' : _EXPLAINREQUEST_COLUMNSENTRY,
    '__module__' : 'ml_worker_pb2'
    # @@protoc_insertion_point(class_scope:worker.ExplainRequest.ColumnsEntry)
    })
  ,
  'DESCRIPTOR' : _EXPLAINREQUEST,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.ExplainRequest)
  })
_sym_db.RegisterMessage(ExplainRequest)
_sym_db.RegisterMessage(ExplainRequest.ColumnsEntry)

ExplainTextRequest = _reflection.GeneratedProtocolMessageType('ExplainTextRequest', (_message.Message,), {

  'ColumnsEntry' : _reflection.GeneratedProtocolMessageType('ColumnsEntry', (_message.Message,), {
    'DESCRIPTOR' : _EXPLAINTEXTREQUEST_COLUMNSENTRY,
    '__module__' : 'ml_worker_pb2'
    # @@protoc_insertion_point(class_scope:worker.ExplainTextRequest.ColumnsEntry)
    })
  ,

  'FeatureTypesEntry' : _reflection.GeneratedProtocolMessageType('FeatureTypesEntry', (_message.Message,), {
    'DESCRIPTOR' : _EXPLAINTEXTREQUEST_FEATURETYPESENTRY,
    '__module__' : 'ml_worker_pb2'
    # @@protoc_insertion_point(class_scope:worker.ExplainTextRequest.FeatureTypesEntry)
    })
  ,
  'DESCRIPTOR' : _EXPLAINTEXTREQUEST,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.ExplainTextRequest)
  })
_sym_db.RegisterMessage(ExplainTextRequest)
_sym_db.RegisterMessage(ExplainTextRequest.ColumnsEntry)
_sym_db.RegisterMessage(ExplainTextRequest.FeatureTypesEntry)

ExplainResponse = _reflection.GeneratedProtocolMessageType('ExplainResponse', (_message.Message,), {

  'Explanation' : _reflection.GeneratedProtocolMessageType('Explanation', (_message.Message,), {

    'PerFeatureEntry' : _reflection.GeneratedProtocolMessageType('PerFeatureEntry', (_message.Message,), {
      'DESCRIPTOR' : _EXPLAINRESPONSE_EXPLANATION_PERFEATUREENTRY,
      '__module__' : 'ml_worker_pb2'
      # @@protoc_insertion_point(class_scope:worker.ExplainResponse.Explanation.PerFeatureEntry)
      })
    ,
    'DESCRIPTOR' : _EXPLAINRESPONSE_EXPLANATION,
    '__module__' : 'ml_worker_pb2'
    # @@protoc_insertion_point(class_scope:worker.ExplainResponse.Explanation)
    })
  ,

  'ExplanationsEntry' : _reflection.GeneratedProtocolMessageType('ExplanationsEntry', (_message.Message,), {
    'DESCRIPTOR' : _EXPLAINRESPONSE_EXPLANATIONSENTRY,
    '__module__' : 'ml_worker_pb2'
    # @@protoc_insertion_point(class_scope:worker.ExplainResponse.ExplanationsEntry)
    })
  ,
  'DESCRIPTOR' : _EXPLAINRESPONSE,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.ExplainResponse)
  })
_sym_db.RegisterMessage(ExplainResponse)
_sym_db.RegisterMessage(ExplainResponse.Explanation)
_sym_db.RegisterMessage(ExplainResponse.Explanation.PerFeatureEntry)
_sym_db.RegisterMessage(ExplainResponse.ExplanationsEntry)

ExplainTextResponse = _reflection.GeneratedProtocolMessageType('ExplainTextResponse', (_message.Message,), {

  'ExplanationsEntry' : _reflection.GeneratedProtocolMessageType('ExplanationsEntry', (_message.Message,), {
    'DESCRIPTOR' : _EXPLAINTEXTRESPONSE_EXPLANATIONSENTRY,
    '__module__' : 'ml_worker_pb2'
    # @@protoc_insertion_point(class_scope:worker.ExplainTextResponse.ExplanationsEntry)
    })
  ,
  'DESCRIPTOR' : _EXPLAINTEXTRESPONSE,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.ExplainTextResponse)
  })
_sym_db.RegisterMessage(ExplainTextResponse)
_sym_db.RegisterMessage(ExplainTextResponse.ExplanationsEntry)

RunModelForDataFrameResponse = _reflection.GeneratedProtocolMessageType('RunModelForDataFrameResponse', (_message.Message,), {
  'DESCRIPTOR' : _RUNMODELFORDATAFRAMERESPONSE,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.RunModelForDataFrameResponse)
  })
_sym_db.RegisterMessage(RunModelForDataFrameResponse)

DataRow = _reflection.GeneratedProtocolMessageType('DataRow', (_message.Message,), {

  'ColumnsEntry' : _reflection.GeneratedProtocolMessageType('ColumnsEntry', (_message.Message,), {
    'DESCRIPTOR' : _DATAROW_COLUMNSENTRY,
    '__module__' : 'ml_worker_pb2'
    # @@protoc_insertion_point(class_scope:worker.DataRow.ColumnsEntry)
    })
  ,
  'DESCRIPTOR' : _DATAROW,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.DataRow)
  })
_sym_db.RegisterMessage(DataRow)
_sym_db.RegisterMessage(DataRow.ColumnsEntry)

DataFrame = _reflection.GeneratedProtocolMessageType('DataFrame', (_message.Message,), {
  'DESCRIPTOR' : _DATAFRAME,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.DataFrame)
  })
_sym_db.RegisterMessage(DataFrame)

RunModelForDataFrameRequest = _reflection.GeneratedProtocolMessageType('RunModelForDataFrameRequest', (_message.Message,), {

  'FeatureTypesEntry' : _reflection.GeneratedProtocolMessageType('FeatureTypesEntry', (_message.Message,), {
    'DESCRIPTOR' : _RUNMODELFORDATAFRAMEREQUEST_FEATURETYPESENTRY,
    '__module__' : 'ml_worker_pb2'
    # @@protoc_insertion_point(class_scope:worker.RunModelForDataFrameRequest.FeatureTypesEntry)
    })
  ,

  'ColumnTypesEntry' : _reflection.GeneratedProtocolMessageType('ColumnTypesEntry', (_message.Message,), {
    'DESCRIPTOR' : _RUNMODELFORDATAFRAMEREQUEST_COLUMNTYPESENTRY,
    '__module__' : 'ml_worker_pb2'
    # @@protoc_insertion_point(class_scope:worker.RunModelForDataFrameRequest.ColumnTypesEntry)
    })
  ,
  'DESCRIPTOR' : _RUNMODELFORDATAFRAMEREQUEST,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.RunModelForDataFrameRequest)
  })
_sym_db.RegisterMessage(RunModelForDataFrameRequest)
_sym_db.RegisterMessage(RunModelForDataFrameRequest.FeatureTypesEntry)
_sym_db.RegisterMessage(RunModelForDataFrameRequest.ColumnTypesEntry)

RunModelRequest = _reflection.GeneratedProtocolMessageType('RunModelRequest', (_message.Message,), {
  'DESCRIPTOR' : _RUNMODELREQUEST,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.RunModelRequest)
  })
_sym_db.RegisterMessage(RunModelRequest)

RunModelResponse = _reflection.GeneratedProtocolMessageType('RunModelResponse', (_message.Message,), {
  'DESCRIPTOR' : _RUNMODELRESPONSE,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.RunModelResponse)
  })
_sym_db.RegisterMessage(RunModelResponse)

RunTestRequest = _reflection.GeneratedProtocolMessageType('RunTestRequest', (_message.Message,), {
  'DESCRIPTOR' : _RUNTESTREQUEST,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.RunTestRequest)
  })
_sym_db.RegisterMessage(RunTestRequest)

SerializedGiskardDataset = _reflection.GeneratedProtocolMessageType('SerializedGiskardDataset', (_message.Message,), {

  'FeatureTypesEntry' : _reflection.GeneratedProtocolMessageType('FeatureTypesEntry', (_message.Message,), {
    'DESCRIPTOR' : _SERIALIZEDGISKARDDATASET_FEATURETYPESENTRY,
    '__module__' : 'ml_worker_pb2'
    # @@protoc_insertion_point(class_scope:worker.SerializedGiskardDataset.FeatureTypesEntry)
    })
  ,

  'ColumnTypesEntry' : _reflection.GeneratedProtocolMessageType('ColumnTypesEntry', (_message.Message,), {
    'DESCRIPTOR' : _SERIALIZEDGISKARDDATASET_COLUMNTYPESENTRY,
    '__module__' : 'ml_worker_pb2'
    # @@protoc_insertion_point(class_scope:worker.SerializedGiskardDataset.ColumnTypesEntry)
    })
  ,
  'DESCRIPTOR' : _SERIALIZEDGISKARDDATASET,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.SerializedGiskardDataset)
  })
_sym_db.RegisterMessage(SerializedGiskardDataset)
_sym_db.RegisterMessage(SerializedGiskardDataset.FeatureTypesEntry)
_sym_db.RegisterMessage(SerializedGiskardDataset.ColumnTypesEntry)

RunTestResponse = _reflection.GeneratedProtocolMessageType('RunTestResponse', (_message.Message,), {
  'DESCRIPTOR' : _RUNTESTRESPONSE,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.RunTestResponse)
  })
_sym_db.RegisterMessage(RunTestResponse)

Partial_unexpected_counts = _reflection.GeneratedProtocolMessageType('Partial_unexpected_counts', (_message.Message,), {
  'DESCRIPTOR' : _PARTIAL_UNEXPECTED_COUNTS,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.Partial_unexpected_counts)
  })
_sym_db.RegisterMessage(Partial_unexpected_counts)

NamedSingleTestResult = _reflection.GeneratedProtocolMessageType('NamedSingleTestResult', (_message.Message,), {
  'DESCRIPTOR' : _NAMEDSINGLETESTRESULT,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.NamedSingleTestResult)
  })
_sym_db.RegisterMessage(NamedSingleTestResult)

TestMessage = _reflection.GeneratedProtocolMessageType('TestMessage', (_message.Message,), {
  'DESCRIPTOR' : _TESTMESSAGE,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.TestMessage)
  })
_sym_db.RegisterMessage(TestMessage)

SingleTestResult = _reflection.GeneratedProtocolMessageType('SingleTestResult', (_message.Message,), {

  'PropsEntry' : _reflection.GeneratedProtocolMessageType('PropsEntry', (_message.Message,), {
    'DESCRIPTOR' : _SINGLETESTRESULT_PROPSENTRY,
    '__module__' : 'ml_worker_pb2'
    # @@protoc_insertion_point(class_scope:worker.SingleTestResult.PropsEntry)
    })
  ,
  'DESCRIPTOR' : _SINGLETESTRESULT,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.SingleTestResult)
  })
_sym_db.RegisterMessage(SingleTestResult)
_sym_db.RegisterMessage(SingleTestResult.PropsEntry)

TestResultMessage = _reflection.GeneratedProtocolMessageType('TestResultMessage', (_message.Message,), {
  'DESCRIPTOR' : _TESTRESULTMESSAGE,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.TestResultMessage)
  })
_sym_db.RegisterMessage(TestResultMessage)

SerializedGiskardModel = _reflection.GeneratedProtocolMessageType('SerializedGiskardModel', (_message.Message,), {
  'DESCRIPTOR' : _SERIALIZEDGISKARDMODEL,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.SerializedGiskardModel)
  })
_sym_db.RegisterMessage(SerializedGiskardModel)

Chunk = _reflection.GeneratedProtocolMessageType('Chunk', (_message.Message,), {
  'DESCRIPTOR' : _CHUNK,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.Chunk)
  })
_sym_db.RegisterMessage(Chunk)

UploadStatus = _reflection.GeneratedProtocolMessageType('UploadStatus', (_message.Message,), {
  'DESCRIPTOR' : _UPLOADSTATUS,
  '__module__' : 'ml_worker_pb2'
  # @@protoc_insertion_point(class_scope:worker.UploadStatus)
  })
_sym_db.RegisterMessage(UploadStatus)

_MLWORKER = DESCRIPTOR.services_by_name['MLWorker']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\021ai.giskard.workerB\013WorkerProtoP\001\242\002\003WRK'
  _MLWORKERINFO_INSTALLEDPACKAGESENTRY._options = None
  _MLWORKERINFO_INSTALLEDPACKAGESENTRY._serialized_options = b'8\001'
  _EXPLAINREQUEST_COLUMNSENTRY._options = None
  _EXPLAINREQUEST_COLUMNSENTRY._serialized_options = b'8\001'
  _EXPLAINTEXTREQUEST_COLUMNSENTRY._options = None
  _EXPLAINTEXTREQUEST_COLUMNSENTRY._serialized_options = b'8\001'
  _EXPLAINTEXTREQUEST_FEATURETYPESENTRY._options = None
  _EXPLAINTEXTREQUEST_FEATURETYPESENTRY._serialized_options = b'8\001'
  _EXPLAINRESPONSE_EXPLANATION_PERFEATUREENTRY._options = None
  _EXPLAINRESPONSE_EXPLANATION_PERFEATUREENTRY._serialized_options = b'8\001'
  _EXPLAINRESPONSE_EXPLANATIONSENTRY._options = None
  _EXPLAINRESPONSE_EXPLANATIONSENTRY._serialized_options = b'8\001'
  _EXPLAINTEXTRESPONSE_EXPLANATIONSENTRY._options = None
  _EXPLAINTEXTRESPONSE_EXPLANATIONSENTRY._serialized_options = b'8\001'
  _DATAROW_COLUMNSENTRY._options = None
  _DATAROW_COLUMNSENTRY._serialized_options = b'8\001'
  _RUNMODELFORDATAFRAMEREQUEST_FEATURETYPESENTRY._options = None
  _RUNMODELFORDATAFRAMEREQUEST_FEATURETYPESENTRY._serialized_options = b'8\001'
  _RUNMODELFORDATAFRAMEREQUEST_COLUMNTYPESENTRY._options = None
  _RUNMODELFORDATAFRAMEREQUEST_COLUMNTYPESENTRY._serialized_options = b'8\001'
  _SERIALIZEDGISKARDDATASET_FEATURETYPESENTRY._options = None
  _SERIALIZEDGISKARDDATASET_FEATURETYPESENTRY._serialized_options = b'8\001'
  _SERIALIZEDGISKARDDATASET_COLUMNTYPESENTRY._options = None
  _SERIALIZEDGISKARDDATASET_COLUMNTYPESENTRY._serialized_options = b'8\001'
  _SINGLETESTRESULT_PROPSENTRY._options = None
  _SINGLETESTRESULT_PROPSENTRY._serialized_options = b'8\001'
  _TESTMESSAGETYPE._serialized_start=4550
  _TESTMESSAGETYPE._serialized_end=4588
  _UPLOADSTATUSCODE._serialized_start=4590
  _UPLOADSTATUSCODE._serialized_end=4641
  _MLWORKERINFOREQUEST._serialized_start=115
  _MLWORKERINFOREQUEST._serialized_end=159
  _MLWORKERINFO._serialized_start=162
  _MLWORKERINFO._serialized_end=485
  _MLWORKERINFO_INSTALLEDPACKAGESENTRY._serialized_start=429
  _MLWORKERINFO_INSTALLEDPACKAGESENTRY._serialized_end=485
  _PLATFORMINFO._serialized_start=487
  _PLATFORMINFO._serialized_end=601
  _FILEUPLOADREQUEST._serialized_start=603
  _FILEUPLOADREQUEST._serialized_end=713
  _FILEUPLOADMETADATA._serialized_start=715
  _FILEUPLOADMETADATA._serialized_end=749
  _ECHOMSG._serialized_start=751
  _ECHOMSG._serialized_end=773
  _EXPLAINREQUEST._serialized_start=776
  _EXPLAINREQUEST._serialized_end=992
  _EXPLAINREQUEST_COLUMNSENTRY._serialized_start=946
  _EXPLAINREQUEST_COLUMNSENTRY._serialized_end=992
  _EXPLAINTEXTREQUEST._serialized_start=995
  _EXPLAINTEXTREQUEST._serialized_end=1331
  _EXPLAINTEXTREQUEST_COLUMNSENTRY._serialized_start=946
  _EXPLAINTEXTREQUEST_COLUMNSENTRY._serialized_end=992
  _EXPLAINTEXTREQUEST_FEATURETYPESENTRY._serialized_start=1280
  _EXPLAINTEXTREQUEST_FEATURETYPESENTRY._serialized_end=1331
  _EXPLAINRESPONSE._serialized_start=1334
  _EXPLAINRESPONSE._serialized_end=1647
  _EXPLAINRESPONSE_EXPLANATION._serialized_start=1419
  _EXPLAINRESPONSE_EXPLANATION._serialized_end=1557
  _EXPLAINRESPONSE_EXPLANATION_PERFEATUREENTRY._serialized_start=1508
  _EXPLAINRESPONSE_EXPLANATION_PERFEATUREENTRY._serialized_end=1557
  _EXPLAINRESPONSE_EXPLANATIONSENTRY._serialized_start=1559
  _EXPLAINRESPONSE_EXPLANATIONSENTRY._serialized_end=1647
  _EXPLAINTEXTRESPONSE._serialized_start=1650
  _EXPLAINTEXTRESPONSE._serialized_end=1793
  _EXPLAINTEXTRESPONSE_EXPLANATIONSENTRY._serialized_start=1742
  _EXPLAINTEXTRESPONSE_EXPLANATIONSENTRY._serialized_end=1793
  _RUNMODELFORDATAFRAMERESPONSE._serialized_start=1796
  _RUNMODELFORDATAFRAMERESPONSE._serialized_end=1937
  _DATAROW._serialized_start=1939
  _DATAROW._serialized_end=2043
  _DATAROW_COLUMNSENTRY._serialized_start=946
  _DATAROW_COLUMNSENTRY._serialized_end=992
  _DATAFRAME._serialized_start=2045
  _DATAFRAME._serialized_end=2087
  _RUNMODELFORDATAFRAMEREQUEST._serialized_start=2090
  _RUNMODELFORDATAFRAMEREQUEST._serialized_end=2479
  _RUNMODELFORDATAFRAMEREQUEST_FEATURETYPESENTRY._serialized_start=1280
  _RUNMODELFORDATAFRAMEREQUEST_FEATURETYPESENTRY._serialized_end=1331
  _RUNMODELFORDATAFRAMEREQUEST_COLUMNTYPESENTRY._serialized_start=2429
  _RUNMODELFORDATAFRAMEREQUEST_COLUMNTYPESENTRY._serialized_end=2479
  _RUNMODELREQUEST._serialized_start=2481
  _RUNMODELREQUEST._serialized_end=2596
  _RUNMODELRESPONSE._serialized_start=2598
  _RUNMODELRESPONSE._serialized_end=2661
  _RUNTESTREQUEST._serialized_start=2664
  _RUNTESTREQUEST._serialized_end=2850
  _SERIALIZEDGISKARDDATASET._serialized_start=2853
  _SERIALIZEDGISKARDDATASET._serialized_end=3171
  _SERIALIZEDGISKARDDATASET_FEATURETYPESENTRY._serialized_start=1280
  _SERIALIZEDGISKARDDATASET_FEATURETYPESENTRY._serialized_end=1331
  _SERIALIZEDGISKARDDATASET_COLUMNTYPESENTRY._serialized_start=2429
  _SERIALIZEDGISKARDDATASET_COLUMNTYPESENTRY._serialized_end=2479
  _RUNTESTRESPONSE._serialized_start=3173
  _RUNTESTRESPONSE._serialized_end=3204
  _PARTIAL_UNEXPECTED_COUNTS._serialized_start=3206
  _PARTIAL_UNEXPECTED_COUNTS._serialized_end=3263
  _NAMEDSINGLETESTRESULT._serialized_start=3265
  _NAMEDSINGLETESTRESULT._serialized_end=3344
  _TESTMESSAGE._serialized_start=3346
  _TESTMESSAGE._serialized_end=3412
  _SINGLETESTRESULT._serialized_start=3415
  _SINGLETESTRESULT._serialized_end=4207
  _SINGLETESTRESULT_PROPSENTRY._serialized_start=4163
  _SINGLETESTRESULT_PROPSENTRY._serialized_end=4207
  _TESTRESULTMESSAGE._serialized_start=4209
  _TESTRESULTMESSAGE._serialized_end=4276
  _SERIALIZEDGISKARDMODEL._serialized_start=4279
  _SERIALIZEDGISKARDMODEL._serialized_end=4466
  _CHUNK._serialized_start=4468
  _CHUNK._serialized_end=4492
  _UPLOADSTATUS._serialized_start=4494
  _UPLOADSTATUS._serialized_end=4548
  _MLWORKER._serialized_start=4644
  _MLWORKER._serialized_end=5191
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import google.protobuf.wrappers_pb2
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _TestMessageType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _TestMessageTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_TestMessageType.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    ERROR: _TestMessageType.ValueType  # 0
    INFO: _TestMessageType.ValueType  # 1

class TestMessageType(_TestMessageType, metaclass=_TestMessageTypeEnumTypeWrapper): ...

ERROR: TestMessageType.ValueType  # 0
INFO: TestMessageType.ValueType  # 1
global___TestMessageType = TestMessageType

class _UploadStatusCode:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _UploadStatusCodeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_UploadStatusCode.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    Unknown: _UploadStatusCode.ValueType  # 0
    Ok: _UploadStatusCode.ValueType  # 1
    Failed: _UploadStatusCode.ValueType  # 2

class UploadStatusCode(_UploadStatusCode, metaclass=_UploadStatusCodeEnumTypeWrapper): ...

Unknown: UploadStatusCode.ValueType  # 0
Ok: UploadStatusCode.ValueType  # 1
Failed: UploadStatusCode.ValueType  # 2
global___UploadStatusCode = UploadStatusCode

class MLWorkerInfoRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LIST_PACKAGES_FIELD_NUMBER: builtins.int
    list_packages: builtins.bool
    def __init__(
        self,
        *,
        list_packages: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["list_packages", b"list_packages"]) -> None: ...

global___MLWorkerInfoRequest = MLWorkerInfoRequest

class MLWorkerInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class InstalledPackagesEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    PLATFORM_FIELD_NUMBER: builtins.int
    INTERPRETER_FIELD_NUMBER: builtins.int
    INTERPRETER_VERSION_FIELD_NUMBER: builtins.int
    INSTALLED_PACKAGES_FIELD_NUMBER: builtins.int
    INTERNAL_GRPC_PORT_FIELD_NUMBER: builtins.int
    IS_REMOTE_FIELD_NUMBER: builtins.int
    PID_FIELD_NUMBER: builtins.int
    PROCESS_START_TIME_FIELD_NUMBER: builtins.int
    @property
    def platform(self) -> global___PlatformInfo: ...
    interpreter: builtins.str
    interpreter_version: builtins.str
    @property
    def installed_packages(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    internal_grpc_port: builtins.int
    is_remote: builtins.bool
    pid: builtins.int
    process_start_time: builtins.int
    def __init__(
        self,
        *,
        platform: global___PlatformInfo | None = ...,
        interpreter: builtins.str = ...,
        interpreter_version: builtins.str = ...,
        installed_packages: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        internal_grpc_port: builtins.int = ...,
        is_remote: builtins.bool = ...,
        pid: builtins.int = ...,
        process_start_time: builtins.int = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["platform", b"platform"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["installed_packages", b"installed_packages", "internal_grpc_port", b"internal_grpc_port", "interpreter", b"interpreter", "interpreter_version", b"interpreter_version", "is_remote", b"is_remote", "pid", b"pid", "platform", b"platform", "process_start_time", b"process_start_time"]) -> None: ...

global___MLWorkerInfo = MLWorkerInfo

class PlatformInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MACHINE_FIELD_NUMBER: builtins.int
    NODE_FIELD_NUMBER: builtins.int
    PROCESSOR_FIELD_NUMBER: builtins.int
    RELEASE_FIELD_NUMBER: builtins.int
    SYSTEM_FIELD_NUMBER: builtins.int
    VERSION_FIELD_NUMBER: builtins.int
    machine: builtins.str
    node: builtins.str
    processor: builtins.str
    release: builtins.str
    system: builtins.str
    version: builtins.str
    def __init__(
        self,
        *,
        machine: builtins.str = ...,
        node: builtins.str = ...,
        processor: builtins.str = ...,
        release: builtins.str = ...,
        system: builtins.str = ...,
        version: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["machine", b"machine", "node", b"node", "processor", b"processor", "release", b"release", "system", b"system", "version", b"version"]) -> None: ...

global___PlatformInfo = PlatformInfo

class FileUploadRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    METADATA_FIELD_NUMBER: builtins.int
    CHUNK_FIELD_NUMBER: builtins.int
    @property
    def metadata(self) -> global___FileUploadMetadata: ...
    @property
    def chunk(self) -> global___Chunk: ...
    def __init__(
        self,
        *,
        metadata: global___FileUploadMetadata | None = ...,
        chunk: global___Chunk | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["chunk", b"chunk", "metadata", b"metadata", "request", b"request"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["chunk", b"chunk", "metadata", b"metadata", "request", b"request"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["request", b"request"]) -> typing_extensions.Literal["metadata", "chunk"] | None: ...

global___FileUploadRequest = FileUploadRequest

class FileUploadMetadata(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    name: builtins.str
    def __init__(
        self,
        *,
        name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["name", b"name"]) -> None: ...

global___FileUploadMetadata = FileUploadMetadata

class EchoMsg(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MSG_FIELD_NUMBER: builtins.int
    msg: builtins.str
    def __init__(
        self,
        *,
        msg: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["msg", b"msg"]) -> None: ...

global___EchoMsg = EchoMsg

class ExplainRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class ColumnsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    MODEL_FIELD_NUMBER: builtins.int
    DATASET_FIELD_NUMBER: builtins.int
    COLUMNS_FIELD_NUMBER: builtins.int
    @property
    def model(self) -> global___SerializedGiskardModel: ...
    @property
    def dataset(self) -> global___SerializedGiskardDataset: ...
    @property
    def columns(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    def __init__(
        self,
        *,
        model: global___SerializedGiskardModel | None = ...,
        dataset: global___SerializedGiskardDataset | None = ...,
        columns: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["dataset", b"dataset", "model", b"model"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["columns", b"columns", "dataset", b"dataset", "model", b"model"]) -> None: ...

global___ExplainRequest = ExplainRequest

class ExplainTextRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class ColumnsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    class FeatureTypesEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    MODEL_FIELD_NUMBER: builtins.int
    FEATURE_NAME_FIELD_NUMBER: builtins.int
    COLUMNS_FIELD_NUMBER: builtins.int
    FEATURE_TYPES_FIELD_NUMBER: builtins.int
    N_SAMPLES_FIELD_NUMBER: builtins.int
    @property
    def model(self) -> global___SerializedGiskardModel: ...
    feature_name: builtins.str
    @property
    def columns(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    @property
    def feature_types(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    n_samples: builtins.int
    def __init__(
        self,
        *,
        model: global___SerializedGiskardModel | None = ...,
        feature_name: builtins.str = ...,
        columns: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        feature_types: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        n_samples: builtins.int = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["model", b"model"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["columns", b"columns", "feature_name", b"feature_name", "feature_types", b"feature_types", "model", b"model", "n_samples", b"n_samples"]) -> None: ...

global___ExplainTextRequest = ExplainTextRequest

class ExplainResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class Explanation(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        class PerFeatureEntry(google.protobuf.message.Message):
            DESCRIPTOR: google.protobuf.descriptor.Descriptor

            KEY_FIELD_NUMBER: builtins.int
            VALUE_FIELD_NUMBER: builtins.int
            key: builtins.str
            value: builtins.float
            def __init__(
                self,
                *,
                key: builtins.str = ...,
                value: builtins.float = ...,
            ) -> None: ...
            def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

        PER_FEATURE_FIELD_NUMBER: builtins.int
        @property
        def per_feature(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.float]: ...
        def __init__(
            self,
            *,
            per_feature: collections.abc.Mapping[builtins.str, builtins.float] | None = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["per_feature", b"per_feature"]) -> None: ...

    class ExplanationsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> global___ExplainResponse.Explanation: ...
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: global___ExplainResponse.Explanation | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    EXPLANATIONS_FIELD_NUMBER: builtins.int
    @property
    def explanations(self) -> google.protobuf.internal.containers.MessageMap[builtins.str, global___ExplainResponse.Explanation]: ...
    def __init__(
        self,
        *,
        explanations: collections.abc.Mapping[builtins.str, global___ExplainResponse.Explanation] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["explanations", b"explanations"]) -> None: ...

global___ExplainResponse = ExplainResponse

class ExplainTextResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class ExplanationsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    EXPLANATIONS_FIELD_NUMBER: builtins.int
    @property
    def explanations(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    def __init__(
        self,
        *,
        explanations: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["explanations", b"explanations"]) -> None: ...

global___ExplainTextResponse = ExplainTextResponse

class RunModelForDataFrameResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ALL_PREDICTIONS_FIELD_NUMBER: builtins.int
    PREDICTION_FIELD_NUMBER: builtins.int
    PROBABILITIES_FIELD_NUMBER: builtins.int
    RAW_PREDICTION_FIELD_NUMBER: builtins.int
    @property
    def all_predictions(self) -> global___DataFrame: ...
    @property
    def prediction(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    @property
    def probabilities(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    @property
    def raw_prediction(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    def __init__(
        self,
        *,
        all_predictions: global___DataFrame | None = ...,
        prediction: collections.abc.Iterable[builtins.str] | None = ...,
        probabilities: collections.abc.Iterable[builtins.float] | None = ...,
        raw_prediction: collections.abc.Iterable[builtins.float] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["all_predictions", b"all_predictions"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["all_predictions", b"all_predictions", "prediction", b"prediction", "probabilities", b"probabilities", "raw_prediction", b"raw_prediction"]) -> None: ...

global___RunModelForDataFrameResponse = RunModelForDataFrameResponse

class DataRow(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class ColumnsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    COLUMNS_FIELD_NUMBER: builtins.int
    @property
    def columns(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    def __init__(
        self,
        *,
        columns: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["columns", b"columns"]) -> None: ...

global___DataRow = DataRow

class DataFrame(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ROWS_FIELD_NUMBER: builtins.int
    @property
    def rows(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___DataRow]: ...
    def __init__(
        self,
        *,
        rows: collections.abc.Iterable[global___DataRow] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["rows", b"rows"]) -> None: ...

global___DataFrame = DataFrame

class RunModelForDataFrameRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class FeatureTypesEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    class ColumnTypesEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    MODEL_FIELD_NUMBER: builtins.int
    DATAFRAME_FIELD_NUMBER: builtins.int
    TARGET_FIELD_NUMBER: builtins.int
    FEATURE_TYPES_FIELD_NUMBER: builtins.int
    COLUMN_TYPES_FIELD_NUMBER: builtins.int
    @property
    def model(self) -> global___SerializedGiskardModel: ...
    @property
    def dataframe(self) -> global___DataFrame: ...
    target: builtins.str
    @property
    def feature_types(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    @property
    def column_types(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    def __init__(
        self,
        *,
        model: global___SerializedGiskardModel | None = ...,
        dataframe: global___DataFrame | None = ...,
        target: builtins.str = ...,
        feature_types: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        column_types: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["dataframe", b"dataframe", "model", b"model"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["column_types", b"column_types", "dataframe", b"dataframe", "feature_types", b"feature_types", "model", b"model", "target", b"target"]) -> None: ...

global___RunModelForDataFrameRequest = RunModelForDataFrameRequest

class RunModelRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MODEL_FIELD_NUMBER: builtins.int
    DATASET_FIELD_NUMBER: builtins.int
    @property
    def model(self) -> global___SerializedGiskardModel: ...
    @property
    def dataset(self) -> global___SerializedGiskardDataset: ...
    def __init__(
        self,
        *,
        model: global___SerializedGiskardModel | None = ...,
        dataset: global___SerializedGiskardDataset | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["dataset", b"dataset", "model", b"model"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["dataset", b"dataset", "model", b"model"]) -> None: ...

global___RunModelRequest = RunModelRequest

class RunModelResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RESULTS_CSV_FIELD_NUMBER: builtins.int
    CALCULATED_CSV_FIELD_NUMBER: builtins.int
    results_csv: builtins.str
    calculated_csv: builtins.str
    def __init__(
        self,
        *,
        results_csv: builtins.str = ...,
        calculated_csv: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["calculated_csv", b"calculated_csv", "results_csv", b"results_csv"]) -> None: ...

global___RunModelResponse = RunModelResponse

class RunTestRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CODE_FIELD_NUMBER: builtins.int
    MODEL_FIELD_NUMBER: builtins.int
    ACTUAL_DS_FIELD_NUMBER: builtins.int
    REFERENCE_DS_FIELD_NUMBER: builtins.int
    code: builtins.str
    @property
    def model(self) -> global___SerializedGiskardModel: ...
    @property
    def actual_ds(self) -> global___SerializedGiskardDataset: ...
    @property
    def reference_ds(self) -> global___SerializedGiskardDataset: ...
    def __init__(
        self,
        *,
        code: builtins.str = ...,
        model: global___SerializedGiskardModel | None = ...,
        actual_ds: global___SerializedGiskardDataset | None = ...,
        reference_ds: global___SerializedGiskardDataset | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["actual_ds", b"actual_ds", "model", b"model", "reference_ds", b"reference_ds"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["actual_ds", b"actual_ds", "code", b"code", "model", b"model", "reference_ds", b"reference_ds"]) -> None: ...

global___RunTestRequest = RunTestRequest

class SerializedGiskardDataset(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class FeatureTypesEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    class ColumnTypesEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    TARGET_FIELD_NUMBER: builtins.int
    FEATURE_TYPES_FIELD_NUMBER: builtins.int
    COLUMN_TYPES_FIELD_NUMBER: builtins.int
    SERIALIZED_DF_FIELD_NUMBER: builtins.int
    target: builtins.str
    @property
    def feature_types(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    @property
    def column_types(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    serialized_df: builtins.bytes
    def __init__(
        self,
        *,
        target: builtins.str = ...,
        feature_types: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        column_types: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        serialized_df: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["column_types", b"column_types", "feature_types", b"feature_types", "serialized_df", b"serialized_df", "target", b"target"]) -> None: ...

global___SerializedGiskardDataset = SerializedGiskardDataset

class RunTestResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    name: builtins.str
    def __init__(
        self,
        *,
        name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["name", b"name"]) -> None: ...

global___RunTestResponse = RunTestResponse

class Partial_unexpected_counts(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALUE_FIELD_NUMBER: builtins.int
    COUNT_FIELD_NUMBER: builtins.int
    @property
    def value(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    count: builtins.int
    def __init__(
        self,
        *,
        value: collections.abc.Iterable[builtins.int] | None = ...,
        count: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["count", b"count", "value", b"value"]) -> None: ...

global___Partial_unexpected_counts = Partial_unexpected_counts

class NamedSingleTestResult(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    RESULT_FIELD_NUMBER: builtins.int
    name: builtins.str
    @property
    def result(self) -> global___SingleTestResult: ...
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        result: global___SingleTestResult | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["result", b"result"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["name", b"name", "result", b"result"]) -> None: ...

global___NamedSingleTestResult = NamedSingleTestResult

class TestMessage(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TYPE_FIELD_NUMBER: builtins.int
    TEXT_FIELD_NUMBER: builtins.int
    type: global___TestMessageType.ValueType
    text: builtins.str
    def __init__(
        self,
        *,
        type: global___TestMessageType.ValueType = ...,
        text: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["text", b"text", "type", b"type"]) -> None: ...

global___TestMessage = TestMessage

class SingleTestResult(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class PropsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    PASSED_FIELD_NUMBER: builtins.int
    MESSAGES_FIELD_NUMBER: builtins.int
    PROPS_FIELD_NUMBER: builtins.int
    METRIC_FIELD_NUMBER: builtins.int
    MISSING_COUNT_FIELD_NUMBER: builtins.int
    MISSING_PERCENT_FIELD_NUMBER: builtins.int
    UNEXPECTED_COUNT_FIELD_NUMBER: builtins.int
    UNEXPECTED_PERCENT_FIELD_NUMBER: builtins.int
    UNEXPECTED_PERCENT_TOTAL_FIELD_NUMBER: builtins.int
    UNEXPECTED_PERCENT_NONMISSING_FIELD_NUMBER: builtins.int
    PARTIAL_UNEXPECTED_INDEX_LIST_FIELD_NUMBER: builtins.int
    PARTIAL_UNEXPECTED_COUNTS_FIELD_NUMBER: builtins.int
    UNEXPECTED_INDEX_LIST_FIELD_NUMBER: builtins.int
    OUTPUT_DF_FIELD_NUMBER: builtins.int
    NUMBER_OF_PERTURBED_ROWS_FIELD_NUMBER: builtins.int
    ACTUAL_SLICES_SIZE_FIELD_NUMBER: builtins.int
    REFERENCE_SLICES_SIZE_FIELD_NUMBER: builtins.int
    passed: builtins.bool
    @property
    def messages(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___TestMessage]: ...
    @property
    def props(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    metric: builtins.float
    @property
    def missing_count(self) -> google.protobuf.wrappers_pb2.Int32Value: ...
    @property
    def missing_percent(self) -> google.protobuf.wrappers_pb2.DoubleValue: ...
    @property
    def unexpected_count(self) -> google.protobuf.wrappers_pb2.Int32Value: ...
    @property
    def unexpected_percent(self) -> google.protobuf.wrappers_pb2.DoubleValue: ...
    @property
    def unexpected_percent_total(self) -> google.protobuf.wrappers_pb2.DoubleValue: ...
    @property
    def unexpected_percent_nonmissing(self) -> google.protobuf.wrappers_pb2.DoubleValue: ...
    @property
    def partial_unexpected_index_list(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    @property
    def partial_unexpected_counts(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Partial_unexpected_counts]: ...
    @property
    def unexpected_index_list(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    output_df: builtins.bytes
    number_of_perturbed_rows: builtins.int
    @property
    def actual_slices_size(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    @property
    def reference_slices_size(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    def __init__(
        self,
        *,
        passed: builtins.bool = ...,
        messages: collections.abc.Iterable[global___TestMessage] | None = ...,
        props: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        metric: builtins.float = ...,
        missing_count: google.protobuf.wrappers_pb2.Int32Value | None = ...,
        missing_percent: google.protobuf.wrappers_pb2.DoubleValue | None = ...,
        unexpected_count: google.protobuf.wrappers_pb2.Int32Value | None = ...,
        unexpected_percent: google.protobuf.wrappers_pb2.DoubleValue | None = ...,
        unexpected_percent_total: google.protobuf.wrappers_pb2.DoubleValue | None = ...,
        unexpected_percent_nonmissing: google.protobuf.wrappers_pb2.DoubleValue | None = ...,
        partial_unexpected_index_list: collections.abc.Iterable[builtins.int] | None = ...,
        partial_unexpected_counts: collections.abc.Iterable[global___Partial_unexpected_counts] | None = ...,
        unexpected_index_list: collections.abc.Iterable[builtins.int] | None = ...,
        output_df: builtins.bytes = ...,
        number_of_perturbed_rows: builtins.int = ...,
        actual_slices_size: collections.abc.Iterable[builtins.int] | None = ...,
        reference_slices_size: collections.abc.Iterable[builtins.int] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["missing_count", b"missing_count", "missing_percent", b"missing_percent", "unexpected_count", b"unexpected_count", "unexpected_percent", b"unexpected_percent", "unexpected_percent_nonmissing", b"unexpected_percent_nonmissing", "unexpected_percent_total", b"unexpected_percent_total"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["actual_slices_size", b"actual_slices_size", "messages", b"messages", "metric", b"metric", "missing_count", b"missing_count", "missing_percent", b"missing_percent", "number_of_perturbed_rows", b"number_of_perturbed_rows", "output_df", b"output_df", "partial_unexpected_counts", b"partial_unexpected_counts", "partial_unexpected_index_list", b"partial_unexpected_index_list", "passed", b"passed", "props", b"props", "reference_slices_size", b"reference_slices_size", "unexpected_count", b"unexpected_count", "unexpected_index_list", b"unexpected_index_list", "unexpected_percent", b"unexpected_percent", "unexpected_percent_nonmissing", b"unexpected_percent_nonmissing", "unexpected_percent_total", b"unexpected_percent_total"]) -> None: ...

global___SingleTestResult = SingleTestResult

class TestResultMessage(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RESULTS_FIELD_NUMBER: builtins.int
    @property
    def results(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___NamedSingleTestResult]: ...
    def __init__(
        self,
        *,
        results: collections.abc.Iterable[global___NamedSingleTestResult] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["results", b"results"]) -> None: ...

global___TestResultMessage = TestResultMessage

class SerializedGiskardModel(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SERIALIZED_PREDICTION_FUNCTION_FIELD_NUMBER: builtins.int
    MODEL_TYPE_FIELD_NUMBER: builtins.int
    THRESHOLD_FIELD_NUMBER: builtins.int
    FEATURE_NAMES_FIELD_NUMBER: builtins.int
    CLASSIFICATION_LABELS_FIELD_NUMBER: builtins.int
    serialized_prediction_function: builtins.bytes
    model_type: builtins.str
    @property
    def threshold(self) -> google.protobuf.wrappers_pb2.DoubleValue: ...
    @property
    def feature_names(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    @property
    def classification_labels(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        serialized_prediction_function: builtins.bytes = ...,
        model_type: builtins.str = ...,
        threshold: google.protobuf.wrappers_pb2.DoubleValue | None = ...,
        feature_names: collections.abc.Iterable[builtins.str] | None = ...,
        classification_labels: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["threshold", b"threshold"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["classification_labels", b"classification_labels", "feature_names", b"feature_names", "model_type", b"model_type", "serialized_prediction_function", b"serialized_prediction_function", "threshold", b"threshold"]) -> None: ...

global___SerializedGiskardModel = SerializedGiskardModel

class Chunk(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CONTENT_FIELD_NUMBER: builtins.int
    content: builtins.bytes
    def __init__(
        self,
        *,
        content: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["content", b"content"]) -> None: ...

global___Chunk = Chunk

class UploadStatus(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CODE_FIELD_NUMBER: builtins.int
    code: global___UploadStatusCode.ValueType
    def __init__(
        self,
        *,
        code: global___UploadStatusCode.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["code", b"code"]) -> None: ...

global___UploadStatus = UploadStatus



import asyncio
import hashlib
import logging
import os
import sys

from daemon import DaemonContext
from daemon.runner import is_pidfile_stale
from lockfile.pidlockfile import PIDLockFile

from giskard.ml_worker.ml_worker import start_ml_worker
from giskard.settings import run_dir

logger = logging.getLogger(__name__)


def remove_stale_pid_file(pid_file):
    if is_pidfile_stale(pid_file):
        logger.debug("Stale PID file found, removing it")
        pid_file.break_lock()


def create_pid_file_path(is_server, host, port):
    key = f"{sys.executable}"
    if not is_server:
        key += f"{host}{port}"
    hash_value = hashlib.sha1(key.encode()).hexdigest()
    return run_dir / f"ml-worker-{hash_value}.pid"


def run_daemon(is_server, host, port):
    log_path = run_dir / "ml-worker.log"
    logger.info(f"Writing logs to {log_path}")
    pid_file = PIDLockFile(create_pid_file_path(is_server, host, port))

    with DaemonContext(pidfile=pid_file, stdout=open(log_path, "w+t")):
        logger.info(f"Daemon PID: {os.getpid()}")
        asyncio.get_event_loop().run_until_complete(start_ml_worker(is_server, host, port))



from typing import List

from giskard.ml_worker.generated.ml_worker_pb2 import NamedSingleTestResult, SingleTestResult
from giskard.ml_worker.testing.drift_tests import DriftTests
from giskard.ml_worker.testing.heuristic_tests import HeuristicTests
from giskard.ml_worker.testing.metamorphic_tests import MetamorphicTests
from giskard.ml_worker.testing.performance_tests import PerformanceTests

EMPTY_SINGLE_TEST_RESULT = SingleTestResult()


class GiskardTestFunctions:
    tests_results: List[NamedSingleTestResult]
    metamorphic: MetamorphicTests
    heuristic: HeuristicTests
    performance: PerformanceTests

    def __init__(self) -> None:
        self.tests_results = []
        self.metamorphic = MetamorphicTests(self.tests_results)
        self.heuristic = HeuristicTests(self.tests_results)
        self.performance = PerformanceTests(self.tests_results)
        self.drift = DriftTests(self.tests_results)



import asyncio
import logging

import grpc

from giskard.settings import settings

logger = logging.getLogger(__name__)


async def _start_grpc_server(is_server=False):
    from giskard.ml_worker.generated.ml_worker_pb2_grpc import add_MLWorkerServicer_to_server
    from giskard.ml_worker.server.ml_worker_service import MLWorkerServiceImpl
    from giskard.ml_worker.utils.network import find_free_port

    server = grpc.aio.server(
        # interceptors=[ErrorInterceptor()],
        options=[
            ("grpc.max_send_message_length", settings.max_send_message_length_mb * 1024 ** 2),
            ("grpc.max_receive_message_length", settings.max_receive_message_length_mb * 1024 ** 2),
        ]
    )

    port = settings.port or find_free_port()
    add_MLWorkerServicer_to_server(MLWorkerServiceImpl(port, not is_server), server)
    server.add_insecure_port(f"{settings.host}:{port}")
    await server.start()
    logger.info(f"Started ML Worker server on port {port}")
    logger.debug(f"ML Worker settings: {settings}")
    return server, port


async def start_ml_worker(is_server=False, remote_host=None, remote_port=None):
    from giskard.ml_worker.bridge.ml_worker_bridge import MLWorkerBridge

    tasks = []
    server, grpc_server_port = await _start_grpc_server(is_server)
    if not is_server:
        logger.info(
            "Remote server host and port are specified, connecting as an external ML Worker"
        )
        tunnel = MLWorkerBridge(grpc_server_port, remote_host, remote_port)
        tasks.append(asyncio.create_task(tunnel.start()))

    tasks.append(asyncio.create_task(server.wait_for_termination()))
    await asyncio.wait(tasks)



import asyncio
import logging
import os

import click
import lockfile
import psutil
from click import INT, STRING
from lockfile.pidlockfile import PIDLockFile, read_pid_from_pidfile, remove_existing_pidfile

from giskard.cli_utils import create_pid_file_path, remove_stale_pid_file, run_daemon
from giskard.ml_worker.ml_worker import start_ml_worker
from giskard.settings import run_dir

run_dir.mkdir(parents=True, exist_ok=True)

logger = logging.getLogger(__name__)


def set_verbose(_ctx, _param, value):
    if value:
        logging.getLogger().setLevel(logging.DEBUG)


@click.group("cli")
def cli():
    """
    Giskard Command Line
    """


@cli.group("worker", help="ML Worker management", context_settings={"show_default": True})
def worker() -> None:
    """
    ML Worker management
    """


def start_stop_options(fn):
    fn = click.option(
        "--host", "-h", type=STRING, default='localhost', help="Remote Giskard host address to connect to"
    )(fn)

    fn = click.option(
        "--server", "-s", "is_server", is_flag=True, default=False,
        help="Server mode. Used by Giskard embedded ML Worker"
    )(fn)
    fn = click.option(
        "--port",
        "-p",
        type=INT,
        default=40051,
        help="Remote Giskard port accepting external ML Worker connections",
    )(fn)
    fn = click.option(
        "--verbose",
        "-v",
        is_flag=True,
        callback=set_verbose,
        default=False,
        expose_value=False,
        is_eager=True,
        help="Enable verbose logging",
    )(fn)
    return fn


@worker.command("start")
@start_stop_options
@click.option(
    "--daemon",
    "-d",
    "is_daemon",
    is_flag=True,
    default=False,
    help="Should ML Worker be started as a Daemon in a background",
)
def start_command(host, port, is_server, is_daemon):
    """\b
    Start ML Worker.

    ML Worker can be started in 2 modes:

    - server: used by default by an ML Worker shipped by Giskard. ML Worker acts as a server that Giskard connects to.

    - client: ML Worker acts as a client and should connect to a running Giskard instance
        by specifying this instance's host and port.
    """

    _start_command(is_server, host, port, is_daemon)


def _start_command(is_server, host, port, is_daemon):
    start_msg = "Starting ML Worker"
    start_msg += " server" if is_server else " client"
    if is_daemon:
        start_msg += " daemon"
    logger.info(start_msg)
    pid_file_path = create_pid_file_path(is_server, host, port)
    pid_file = PIDLockFile(pid_file_path)
    remove_stale_pid_file(pid_file)
    try:
        pid_file.acquire()
        if is_daemon:
            # Releasing the lock because it will be re-acquired by a daemon process
            pid_file.release()
            run_daemon(is_server, host, port)
        else:
            asyncio.get_event_loop().run_until_complete(start_ml_worker(is_server, host, port))
    except KeyboardInterrupt:
        logger.info("Exiting")
    except lockfile.AlreadyLocked:
        existing_pid = read_pid_from_pidfile(pid_file_path)
        logger.warning(
            f"Another ML Worker {_ml_worker_description(is_server, host, port)} "
            f"is already running with PID: {existing_pid}. "
            f"Not starting a new one."
        )
    finally:
        if pid_file.i_am_locking():
            pid_file.release()


def _ml_worker_description(is_server, host, port):
    return "server" if is_server else f"client for {host or 'localhost'}:{port or ''}"


@worker.command("stop", help="Stop running ML Workers")
@start_stop_options
@click.option(
    "--all", "-a", "stop_all", is_flag=True, default=False, help="Stop all running ML Workers"
)
def stop_command(is_server, host, port, stop_all):
    import re

    if stop_all:
        for pid_fname in os.listdir(run_dir):
            if not re.match(r"^ml-worker-.*\.pid$", pid_fname):
                continue
            _stop_pid_fname(pid_fname)
    else:
        _find_and_stop(is_server, host, port)


@worker.command("restart", help="Restart ML Worker")
@start_stop_options
def restart_command(is_server, host, port):
    _find_and_stop(is_server, host, port)
    _start_command(is_server, host, port, is_daemon=True)


def _stop_pid_fname(pid_fname):
    pid_file_path = str(run_dir / pid_fname)
    remove_stale_pid_file(PIDLockFile(pid_file_path))
    pid = read_pid_from_pidfile(pid_file_path)
    if pid:
        worker_process = psutil.Process(pid)
        worker_process.terminate()
        logger.info(f"Stopped ML Worker Daemon by PID: {pid}")
    remove_existing_pidfile(pid_file_path)


def _find_and_stop(is_server, host, port):
    pid_file_path = str(create_pid_file_path(is_server, host, port))
    remove_stale_pid_file(PIDLockFile(pid_file_path))
    pid = read_pid_from_pidfile(pid_file_path)
    logger.info("Stopping ML Worker Daemon")
    if pid:
        worker_process = psutil.Process(pid)
        worker_process.terminate()
        logger.info(f"Stopped ML Worker {_ml_worker_description(is_server, host, port)}")
    else:
        logger.info(f"ML Worker {_ml_worker_description(is_server, host, port)} is not running")
    remove_existing_pidfile(pid_file_path)


if __name__ == "__main__":
    cli()



from typing import Any, Callable, Iterable, List, Optional, Union

import logging
from builtins import Exception

import numpy
import pandas as pd
from pydantic import BaseModel

from giskard.ml_worker.core.giskard_dataset import GiskardDataset
from giskard.ml_worker.utils.logging import Timer

logger = logging.getLogger(__name__)


class ModelPredictionResults(BaseModel):
    prediction: Any
    raw_prediction: Any
    probabilities: Optional[Any]
    all_predictions: Optional[Any]


class GiskardModel:
    def __init__(
        self,
        prediction_function: Callable[[pd.DataFrame], Iterable[Union[str, float, int]]],
        model_type: str,
        feature_names: Optional[List[str]],
        classification_threshold: float = None,
        classification_labels: List[str] = None,
    ) -> None:
        self.prediction_function = prediction_function
        self.model_type = model_type
        self.classification_threshold = classification_threshold
        self.feature_names = feature_names
        self.classification_labels = classification_labels

    def run_predict(self, dataset: GiskardDataset):
        timer = Timer()
        df = self.prepare_dataframe(dataset)
        raw_prediction = self.prediction_function(df)
        if self.model_type == "regression":
            result = ModelPredictionResults(
                prediction=raw_prediction, raw_prediction=raw_prediction
            )
        elif self.model_type == "classification":
            labels = numpy.array(self.classification_labels)
            threshold = self.classification_threshold

            if threshold is not None and len(labels) == 2:
                predicted_lbl_idx = (raw_prediction[:, 1] > threshold).astype(int)
            else:
                predicted_lbl_idx = raw_prediction.argmax(axis=1)

            all_predictions = pd.DataFrame(raw_prediction, columns=labels)

            predicted_labels = labels[predicted_lbl_idx]
            probability = raw_prediction[range(len(predicted_lbl_idx)), predicted_lbl_idx]

            result = ModelPredictionResults(
                prediction=predicted_labels,
                raw_prediction=predicted_lbl_idx,
                probabilities=probability,
                all_predictions=all_predictions,
            )
        else:
            raise ValueError(f"Prediction task is not supported: {self.model_type}")
        timer.stop(f"Predicted dataset with shape {dataset.df.shape}")
        return result

    def prepare_dataframe(self, dataset):
        df = dataset.df.copy()
        column_types = dict(dataset.column_types) if dataset.column_types else None
        if dataset.target:
            if dataset.target in df.columns:
                df.drop(dataset.target, axis=1, inplace=True)
            if column_types and dataset.target in column_types:
                del column_types[dataset.target]

        if self.feature_names:
            if set(self.feature_names) > set(df.columns):
                column_names = set(self.feature_names) - set(df.columns)
                raise ValueError(
                    f"The following columns are not found in the dataset: {', '.join(sorted(column_names))}"
                )
            df = df[self.feature_names]
            if column_types:
                column_types = {k: v for k, v in column_types.items() if k in self.feature_names}

        if column_types:
            df = self.cast_column_to_types(df, column_types)
        return df

    @staticmethod
    def cast_column_to_types(df, column_types):
        current_types = df.dtypes.apply(lambda x: x.name).to_dict()
        logger.info(f"Casting dataframe columns from {current_types} to {column_types}")
        if column_types:
            try:
                df = df.astype(column_types)
            except Exception as e:
                raise ValueError("Failed to apply column types to dataset") from e
        return df



# type: ignore[attr-defined]
"""Inspect your AI models visually, find bugs, give feedback 🕵️‍♀️ 💬"""

import sys

from giskard.client.giskard_client import GiskardClient
from giskard.ml_worker.utils.logging import configure_logging

configure_logging()

if sys.version_info >= (3, 8):
    from importlib import metadata as importlib_metadata
else:
    import importlib_metadata


def get_version() -> str:
    try:
        return importlib_metadata.version(__name__)
    except importlib_metadata.PackageNotFoundError:  # pragma: no cover
        return "unknown"


__version__: str = get_version()
